---
id: architecture
---
# Project Organization and Best Practices

This document outlines the organization and best practices for the Bluefly AI Platform projects, emphasizing their status as separate, independent projects rather than a monorepo.

## Project Structure

Each project in `.` is a standalone npm package with its own:
- Version control
- Dependencies
- Build process
- Testing framework
- Documentation
- Deployment pipeline

### Core AI Projects

1. **@bluefly/llm** (`bfllm/`)
   - LLM inference and training services
   - vLLM integration for model serving
   - TensorFlow.js for ML operations
   - MLflow for experiment tracking
   - OpenTelemetry for observability

2. **@bluefly/mcp** (`bfmcp/`)
   - Model Context Protocol server
   - CrewAI agent orchestration
   - Qdrant vector database integration
   - LangChain tool integration
   - MCP 2025 protocol support

3. **@bluefly/native-ai** (`native_aijs/`)
   - Neural pattern recognition
   - Content analysis
   - Intelligent evolution
   - Pattern learning

### Platform Projects

4. **@bluefly/api** (`bfapi/`)
   - API Gateway
   - OpenAPI 3.1 compliance
   - Authentication/Authorization
   - Rate limiting
   - Request validation

5. **@bluefly/cli** (`bfcli/`)
   - Command-line interface
   - Project management
   - Development tools
   - Deployment automation

6. **@bluefly/ui** (`bfui/`)
   - React components
   - TypeScript interfaces
   - Design system
   - Component library

### Integration Projects

7. **@bluefly/drupal-llm** (`llm_platform_recipe/`)
   - Drupal 11 integration
   - Recipe-based deployment
   - Module orchestration
   - Content intelligence

8. **@bluefly/api-client** (`bfapiclient/`)
   - TypeScript API client
   - OpenAPI-generated types
   - Authentication handling
   - Request/response validation

## Project Standards

### 1. Package Configuration

Every project must include:

```json
{
  "name": "@bluefly/<project-name>",
  "version": "1.0.0",
  "type": "module",
  "main": "dist/index.js",
  "types": "dist/index.d.ts",
  "module": "dist/index.mjs",
  "files": [
    "dist/**/*",
    "README.md",
    "CHANGELOG.md"
  ],
  "publishConfig": {
    "registry": "https://registry.npmjs.org/",
    "access": "public"
  },
  "engines": {
    "node": ">=20.0.0",
    "npm": ">=9.0.0"
  }
}
```

### 2. Development Scripts

Required scripts for all projects:

```json
{
  "scripts": {
    "build": "tsup",
    "build:prod": "npm run build",
    "ci": "npm run lint && npm run typecheck && npm run test && npm run build",
    "dev": "tsx src/index.ts",
    "format": "prettier --write \"src/**/*.{ts,js,json}\"",
    "lint": "eslint 'src/**/*.{ts,js}'",
    "test": "jest --passWithNoTests",
    "test:ci": "jest --passWithNoTests --ci",
    "test:coverage": "jest --coverage",
    "typecheck": "tsc --noEmit",
    "validate": "npm run format:check && npm run lint && npm run typecheck && npm run test"
  }
}
```

### 3. Testing Requirements

- Jest for unit and integration tests
- 90%+ test coverage
- OpenAPI validation for APIs
- Performance benchmarks
- Security scanning

### 4. Documentation

Each project must include:

1. **README.md**
   - Project overview
   - Installation
   - Usage examples
   - API reference
   - Contributing guide

2. **CHANGELOG.md**
   - Version history
   - Breaking changes
   - Feature additions
   - Bug fixes

3. **API Documentation**
   - OpenAPI specification
   - TypeScript interfaces
   - Example requests/responses
   - Error handling

### 5. Dependencies

Follow these guidelines:

1. **Production Dependencies**
   - Pin exact versions
   - Regular security updates
   - Minimal dependencies
   - Clear upgrade path

2. **Development Dependencies**
   - Latest stable versions
   - Shared configurations
   - Development tools
   - Testing frameworks

### 6. TypeScript Configuration

```json
{
  "compilerOptions": {
    "target": "ES2022",
    "module": "ESNext",
    "moduleResolution": "bundler",
    "declaration": true,
    "strict": true,
    "skipLibCheck": true,
    "esModuleInterop": true,
    "isolatedModules": true,
    "forceConsistentCasingInFileNames": true
  },
  "include": ["src/**/*"],
  "exclude": ["node_modules", "dist", "**/*.test.ts"]
}
```

### 7. ESLint Configuration

```javascript
// eslint.config.js
export default {
  extends: [
    'eslint:recommended',
    'plugin:@typescript-eslint/recommended',
    'plugin:prettier/recommended'
  ],
  parser: '@typescript-eslint/parser',
  plugins: ['@typescript-eslint'],
  rules: {
    '@typescript-eslint/explicit-function-return-type': 'error',
    '@typescript-eslint/no-explicit-any': 'error',
    '@typescript-eslint/no-unused-vars': 'error'
  }
};
```

### 8. GitLab CI/CD

```yaml
# .gitlab-ci.yml
stages:
  - validate
  - test
  - build
  - deploy

validate:
  script:
    - npm run validate
    - npm run validate:openapi

test:
  script:
    - npm run test:ci
    - npm run test:coverage

build:
  script:
    - npm run build:prod
    - npm run docker:build

deploy:
  script:
    - npm run deploy:prod
```

## Project Dependencies

### Local Development

For local development, use file dependencies:

```json
{
  "dependencies": {
    "@bluefly/cli": "file:../bfcli",
    "@bluefly/api-client": "file:../bfapiclient"
  }
}
```

### Production Dependencies

For production, use published versions:

```json
{
  "dependencies": {
    "@bluefly/cli": "^1.0.0",
    "@bluefly/api-client": "^1.0.0"
  }
}
```

## Version Management

1. **Semantic Versioning**
   - MAJOR: Breaking changes
   - MINOR: New features
   - PATCH: Bug fixes

2. **Release Process**
   ```bash
   # Create release
   npm version <major|minor|patch>
   npm publish
   
   # Update dependencies
   npm update @bluefly/*
   ```

3. **Dependency Updates**
   ```bash
   # Check for updates
   npm outdated
   
   # Update dependencies
   npm update
   
   # Audit security
   npm audit
   ```

## Project Communication

1. **Inter-Project Communication**
   - REST APIs (OpenAPI 3.1)
   - gRPC for internal services
   - Event-driven messaging
   - WebSocket for real-time

2. **Data Flow**
   ```
   Client -> API Gateway -> Service -> Database
                    -> Cache
                    -> Message Queue
   ```

## Security

1. **Authentication**
   - JWT for API access
   - OAuth2 for user auth
   - API keys for services

2. **Authorization**
   - Role-based access
   - Resource permissions
   - Audit logging

3. **Security Scanning**
   - Dependency scanning
   - Code analysis
   - Penetration testing

## Monitoring

1. **Observability**
   - OpenTelemetry
   - Prometheus metrics
   - Grafana dashboards
   - Log aggregation

2. **Health Checks**
   - Service health
   - Dependency status
   - Resource usage
   - Performance metrics

## Best Practices

1. **Code Organization**
   - Clear directory structure
   - Modular design
   - Interface-first approach
   - Documentation-driven

2. **Development Workflow**
   - Feature branches
   - Pull requests
   - Code review
   - Automated testing

3. **Quality Assurance**
   - TypeScript strict mode
   - ESLint rules
   - Test coverage
   - Performance benchmarks

4. **Documentation**
   - API documentation
   - Code comments
   - Architecture diagrams
   - Deployment guides

## Project-Specific Guidelines

### AI/ML Projects

1. **Model Management**
   - Version control for models
   - Experiment tracking
   - Performance metrics
   - A/B testing

2. **Training Pipeline**
   - Data validation
   - Model validation
   - Performance testing
   - Deployment strategy

### API Projects

1. **API Design**
   - OpenAPI 3.1
   - Resource-oriented
   - Version management
   - Error handling

2. **Performance**
   - Response caching
   - Rate limiting
   - Load balancing
   - Circuit breaking

### UI Projects

1. **Component Library**
   - Design system
   - Accessibility
   - Performance
   - Browser support

2. **State Management**
   - Type safety
   - Predictable updates
   - Performance optimization
   - Developer experience 
