---
id: components-bfcicomponents-workflow
---
# GitLab CI/CD Workflow with BFCLI

This document explains how to use the BFCLI workflow commands to manage CI/CD components and automate common GitLab tasks.

## Installation

Make sure you have the BFCLI installed:

```bash
npm install -g @blueflyio/bfcli
```

## CI/CD Component Commands

BFCLI provides dedicated commands for working with CI/CD components in the BFCIComponents repository.

### Validating Components

Validate that all components conform to the required structure and standards:

```bash
# Validate all components
bfcli workflow validate-components

# Validate components in a specific repository
bfcli workflow validate-components --project BFCIComponents
```

The validation checks:

- Proper component format with spec section
- Required inputs definition
- Component metadata
- Custom image support
- Default values for inputs
- Documentation

### Listing Components

List all available CI/CD components:

```bash
# List all components
bfcli workflow list-components

# List components in a specific repository
bfcli workflow list-components --project BFCIComponents
```

The output shows:

- Component name
- File path
- Description
- Whether it has proper metadata

### Publishing Components

Publish a component to the GitLab CI/CD Catalog:

```bash
# Publish a component with automatic versioning
bfcli workflow publish-component nodejs-base

# Publish with a specific tag and message
bfcli workflow publish-component nodejs-base --tag v1.2.0 --message "Add support for Node.js 20"

# Publish to a specific project
bfcli workflow publish-component nodejs-base --project BFCIComponents

# Publish with detailed description
bfcli workflow publish-component nodejs-base --description "This release adds support for Node.js 20 and improves caching"

# Skip validation (not recommended)
bfcli workflow publish-component nodejs-base --no-validate
```

Publishing:

1. Validates the component (unless --no-validate is specified)
2. Creates a git tag
3. Pushes the tag to the remote repository
4. Creates a GitLab release
5. Makes the component available in the CI/CD Catalog

## Other Workflow Commands

BFCLI includes many additional workflow commands for GitLab:

### Merge Request Commands

```bash
# Create a merge request
bfcli workflow mr-create --title "Fix build issue" --description "This fixes the CI build failure"

# List all merge requests
bfcli workflow mr-list

# Update a merge request
bfcli workflow mr-update 123 --title "New title"

# Merge a merge request
bfcli workflow mr-merge 123 --delete-branch

# Close a merge request
bfcli workflow mr-close 123
```

### Pipeline Commands

```bash
# Run a GitLab pipeline
bfcli workflow pipeline-run --branch main

# List pipelines
bfcli workflow pipeline-list --status running
```

### Issue Commands

```bash
# Create an issue
bfcli workflow issue-create --title "Fix documentation" --description "The docs are incomplete"

# List issues
bfcli workflow issue-list
```

### Release Commands

```bash
# Create a release
bfcli workflow release-create --tag v1.0.0 --name "Version 1.0.0" --description "Initial release"
```

## Using in CI/CD Pipelines

The workflow commands can be used in CI/CD pipelines by installing BFCLI in the CI environment:

```yaml
before_script:
  - npm install -g @blueflyio/bfcli
  - bfcli --version

validate-job:
  script:
    - bfcli workflow validate-components
```

## Automation Tips

1. To trigger a component release from CI, include a special prefix in your commit message:

   ```
   release: Update nodejs-base component
   ```

2. To trigger documentation updates, include a special prefix:

   ```
   docs: Update component documentation
   ```

3. For bulk operations across multiple projects, use the `--projects` option:
   ```bash
   bfcli workflow setup-all --projects BFAPI,BFLLM,BFUI
   ```
