---
id: demo_consolidated-CLAUDE
---
# CLAUDE.md

This file provides guidance to Claude Code (claude.ai/code) when working with code in this repository.

## Project Overview

This is a **Drupal 11 AI platform** featuring custom modules for LLM integration, API normalization, and AI-powered workflows. The project combines Drupal's enterprise content management with cutting-edge AI capabilities.

## Key Architecture

### Drupal Core Structure
- **Web root**: `web/` (standard Composer-scaffolded Drupal 11)
- **Custom modules**: `web/modules/custom/` (AI-focused modules)
- **Configuration**: Recipe-based installation via `recipes/llm_platform/`
- **Hybrid stack**: PHP/Drupal backend with TypeScript/Node.js components

### AI Module Ecosystem
The platform consists of specialized modules that work together:

- **api_normalization**: Core API transformation and normalization engine
- **llm**: LLM integration services and model management
- **ai_swarm_intelligence**: Advanced AI coordination and swarm behaviors
- **langchain**: LangChain integration for AI orchestration
- **secure_drupal**: Enterprise security for AI operations
- **field_eca_bridge**: Event-driven field processing automation

### Recipe System
Uses Drupal 11's recipe system for modular installation:
- **Main recipe**: `recipes/llm_platform/recipe.yml`
- **Sub-recipes**: Domain-separated recipes in `recipes/llm_platform/recipes/`
- **Content types**: AI agents, conversations, models, workflows
- **User roles**: AI administrator, editor, data scientist, platform manager

## Development Commands

**⚠️ IMPORTANT: Use DDEV to avoid PHP 8.4 deprecation warnings**

### Drupal Operations (DDEV Recommended)
```bash
# DDEV Environment (PHP 8.3 - No Deprecation Warnings) ✅
ddev start                    # Start DDEV environment
ddev composer install        # Install PHP dependencies
ddev composer update         # Update dependencies
ddev composer require drupal/module # Add Drupal module

# Drush commands (DDEV)
ddev drush cr                 # Clear cache
ddev drush updb               # Run database updates
ddev drush cex                # Export configuration
ddev drush cim                # Import configuration
ddev drush uli                # Generate login link

# Module management (DDEV)
ddev drush en module_name     # Enable module
ddev drush pmu module_name    # Uninstall module
ddev drush pm:list --status=enabled # List enabled modules

# Recipe installation (DDEV)
ddev drush recipe recipes/llm_platform # Install complete AI platform

# Local Commands (PHP 8.4 - Produces Vendor Deprecation Warnings) ⚠️
composer install              # Install PHP dependencies (with warnings)
drush cr                      # Clear cache (with warnings)
drush updb                    # Run database updates (with warnings)
# Note: Warnings are from vendor code (Drush, Symfony, Psy) - not project code
```

**PHP Environment Notes:**
- **DDEV (PHP 8.3.21)**: ✅ No deprecation warnings - **RECOMMENDED**
- **Local (PHP 8.4.8)**: ❌ Produces vendor deprecation warnings - Avoid for development

### Testing Commands
```bash
# Comprehensive test suite
./tests/run-all-tests.sh              # Run all tests
./tests/run-simple-tests.sh           # Quick smoke tests
./tests/run-milvus-tests.sh          # Vector database tests

# Module-specific testing
cd web/modules/custom/MODULE_NAME
vendor/bin/phpunit                    # Run PHPUnit tests

# API testing
curl http://localhost:8080/api/v1/health  # Health check
```

### TypeScript/JavaScript Operations
```bash
# Build TypeScript components
npm run build                 # Compile TypeScript
npm test                     # Run Jest tests

# Linting and formatting
npm run lint                 # ESLint check (uses @bluefly/eslint-config)
npm run format              # Prettier formatting (@bluefly/prettier-config)
```

## Module Development Patterns

### Drupal Module Naming and Structure Conventions

#### Submodule Naming Pattern
All submodules must follow the pattern: `parentmachine_submodulemachine`

Examples:
- `langchain_adapter` (parent: langchain)
- `drupal_migration_plus_wordpress` (parent: drupal_migration_plus)
- `secure_drupal_compliance` (parent: secure_drupal)

#### Module Organization Structure
```
web/modules/custom/
├── langchain/                           # Parent module
│   ├── langchain.info.yml              # Core LangChain integration
│   ├── modules/
│   │   ├── langchain_adapter/          # Unified LLM provider interface
│   │   ├── langchain_rag/              # Retrieval-augmented generation
│   │   ├── langchain_eca/              # Event-driven workflows
│   │   └── langchain_observability/    # Metrics and monitoring
├── drupal_migration_plus/              # Parent module
│   ├── drupal_migration_plus.info.yml  # Core migration framework
│   ├── modules/
│   │   ├── drupal_migration_plus_wordpress/     # WordPress migration
│   │   ├── drupal_migration_plus_sitecore/      # Sitecore migration
│   │   └── drupal_migration_plus_experience_builder/ # Experience Builder
└── secure_drupal/                      # Parent module
    ├── secure_drupal.info.yml          # Core security framework
    ├── modules/
    │   ├── secure_drupal_compliance/   # Compliance framework
    │   ├── secure_drupal_auth/         # Advanced authentication
    │   └── secure_drupal_evidence/     # Evidence collection
```

#### Package Grouping in .info.yml
```yaml
# langchain_rag.info.yml
name: 'LangChain RAG'
type: module
description: 'Retrieval-augmented generation capabilities'
package: 'LangChain'
core_version_requirement: ^11
dependencies:
  - langchain:langchain
```

### Module Architecture Reference

#### Complete Module/Submodule Structure

**LangChain Package:**
- `langchain` - Core orchestration for LLM workflows  
- `langchain_adapter` - Unified LLM provider interface
- `langchain_rag` - Retrieval-augmented generation capabilities
- `langchain_eca` - Event-driven orchestrator workflows
- `langchain_langgraph` - Graph-based multi-agent orchestration
- `langchain_observability` - Metrics/logging/monitoring integration
- `langchain_ai_providers` - Supports BFAPI, OpenAI, Claude, Ollama

**Migration Plus Package:**
- `drupal_migration_plus` - Core migration with AI enhancements
- `drupal_migration_plus_wordpress` - WordPress to Drupal 11 migration
- `drupal_migration_plus_themes` - Theme migration with visual analysis
- `drupal_migration_plus_sitecore` - Sitecore to Drupal 11 migration
- `drupal_migration_plus_html` - Intelligent content scraping
- `drupal_migration_plus_experience_builder` - Experience Builder integration
- `drupal_migration_plus_aem` - Adobe AEM to Drupal migration

**Secure Drupal Package:**
- `secure_drupal` - Hardened security framework
- `secure_drupal_sod` - Separation of duties controls  
- `secure_drupal_policy_bridge` - Policy enforcement bridge
- `secure_drupal_evidence` - Evidence collection tools
- `secure_drupal_compliance` - Compliance framework (GDPR, HIPAA, SOC2)
- `secure_drupal_auth` - Advanced authentication capabilities

**API Normalization Package:**
- `api_normalization_core` - Core data normalization router
- `api_normalization_versioning` - API version management
- `api_normalization_transform` - Payload transformation
- `api_normalization_schema` - Validation and schema generation
- `api_normalization_cache` - Caching layer for normalization
- `api_normalization_eca` - Workflow triggers for API calls

### Module Restructuring Commands
```bash
# After restructuring modules, rebuild cache
drush cr

# Verify module discovery by package
drush pm:list --package="LangChain"
drush pm:list --package="Migration Plus" 
drush pm:list --package="Secure Drupal"
drush pm:list --package="API Normalization"

# Enable parent and submodules
drush en langchain langchain_adapter langchain_rag
drush en drupal_migration_plus drupal_migration_plus_wordpress
drush en secure_drupal secure_drupal_compliance secure_drupal_auth

# Verify dependencies are correctly resolved
drush pm:list --status=enabled --package="LangChain"
```

### Module Development Workflow
```bash
# 1. Create parent module structure
mkdir web/modules/custom/my_module
mkdir web/modules/custom/my_module/modules

# 2. Create submodule following naming convention  
mkdir web/modules/custom/my_module/modules/my_module_subfeature

# 3. Ensure proper .info.yml dependencies
echo "dependencies:
  - my_module:my_module" >> web/modules/custom/my_module/modules/my_module_subfeature/my_module_subfeature.info.yml

# 4. Rebuild cache after structural changes
drush cr
```

### Native Drupal Integration Architecture
The platform follows enterprise-grade patterns proven in the `api_normalization` module:

#### Field API Integration
```php
// Custom field types with comprehensive properties
class CustomFieldItem extends FieldItemBase {
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    // Define 8+ properties with validation
  }
  
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    // Define database schema
  }
}
```

#### Plugin System Architecture
```php
// Plugin manager with discovery and validation
class CustomPluginManager extends DefaultPluginManager {
  // Complete plugin system with annotation discovery
}

// Plugin interface with comprehensive methods
interface CustomPluginInterface {
  // Configuration, validation, transformation methods
}
```

#### Views Integration
```php
// Custom Views field for live data display
class CustomViewsField extends FieldPluginBase {
  public function render(ResultRow $values) {
    // Live API response rendering with caching
  }
}
```

### Service Registration Pattern
```yaml
# module.services.yml
services:
  module.service_name:
    class: Drupal\module\Service\ServiceClass
    arguments: ['@entity_type.manager', '@logger.factory']
    tags:
      - { name: service_tag }
```

### Configuration Management
```bash
# Export module configuration
drush config:export --module=module_name

# Import specific configuration
drush config:import --partial --source=config/install/

# Recipe configuration validation
drush recipe:validate llm_platform --recursive
```

## Testing Architecture

### Test Structure
- **API tests**: `tests/api/` - API endpoint validation
- **Integration tests**: `tests/integration/` - Module interaction testing
- **Performance tests**: `tests/performance/` - Load and benchmark testing
- **Security tests**: `tests/security/` - Security validation

### Module Testing Pattern
Each custom module includes:
- **PHPUnit tests**: `tests/` directory in module
- **Test coverage**: Target 95%+ for innovation score
- **Integration validation**: Cross-module testing

### Test Execution Flow
```bash
# The test runner validates:
# 1. Module status and dependencies
# 2. Database connectivity
# 3. API endpoint responses
# 4. Drush command functionality
# 5. Integration between modules
# 6. Performance benchmarks
# 7. Security compliance
```

## AI Platform Specifics

### LLM Integration Points
- **Model management**: Configuration and performance tracking
- **API normalization**: Transform responses between different AI providers
- **Vector operations**: Semantic search and embeddings
- **Conversation handling**: Threaded AI interactions with context

### AI Content Types
- **AI Agent**: Configurable assistants with specialized roles
- **AI Conversation**: Threaded interactions with full context
- **AI Model**: Model configurations with performance metrics
- **AI Workflow**: Automated processing pipelines

### Security Considerations
- All AI operations require proper authentication
- Rate limiting on API endpoints
- Input validation and sanitization
- Audit logging for AI interactions
- Role-based access control for AI features

### Data Sovereignty and Privacy
- **Private LLM Support**: Platform designed for closed-source, on-premises LLM deployments
- **Data Residency**: All AI processing can be contained within organizational boundaries
- **Zero External Data Leakage**: Configure models to prevent data transmission to external providers
- **Compliance Ready**: Built for GDPR, HIPAA, SOC2, and enterprise data governance requirements
- **Encrypted AI Pipelines**: End-to-end encryption for all AI model interactions
- **Audit Trails**: Complete data lineage tracking for compliance and security audits

## Performance Optimization

### Caching Strategy
```bash
# Enable production caching
drush config:set system.performance css.preprocess 1
drush config:set system.performance js.preprocess 1
drush cr
```

### Database Optimization
- Vector database integration (Milvus) for semantic operations
- Efficient API response caching
- Optimized field storage for AI data

## Development Workflow

### Adding New AI Features
1. Create or extend custom module in `web/modules/custom/`
2. Follow native integration patterns (Field API, Plugin system, Views)
3. Add comprehensive tests with 95%+ coverage
4. Update recipe configuration if needed
5. Run full test suite: `./tests/run-all-tests.sh`

### Code Quality Standards
- Follow Drupal coding standards
- Maintain 95%+ innovation score for Drupal.org readiness
- Comprehensive PHPUnit test coverage
- TypeScript compilation without errors
- ESLint compliance using @bluefly/eslint-config

### Configuration Export/Import
```bash
# After making configuration changes
drush cex                     # Export configuration
git add config/               # Commit configuration changes

# Deploying configuration changes
drush cim                     # Import configuration
drush cr                      # Clear cache
```

This platform represents enterprise-grade AI integration with Drupal 11, following proven patterns for scalability, maintainability, and Drupal.org compliance.
