/**
 * @file
 * JavaScript behaviors for the Native AI.js logs dashboard widget.
 */

(function ($, Drupal) {
  'use strict';

  /**
   * Behavior for Native AI.js logs dashboard widget.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.nativeAiJsLogsDashboard = {
    attach: function (context, settings) {
      // Only run once
      $('.native-aijs-logs-summary', context).once('nativeAiJsLogsDashboard').each(function () {
        const $dashboard = $(this);
        
        // Initialize tooltips for operation bars
        $dashboard.find('.operation-bar').each(function() {
          const $this = $(this);
          Drupal.behaviors.nativeAiJsLogsDashboard.initializeTooltip($this);
        });
        
        // Add click handlers for logs
        $dashboard.find('.log-entry').each(function() {
          const $logRow = $(this);
          const $detailsLink = $logRow.find('.log-details a');
          
          // Make entire row clickable, triggering the details link
          $logRow.on('click', function(e) {
            // Only if we didn't click directly on the link
            if (!$(e.target).is('a') && !$(e.target).parent().is('a')) {
              e.preventDefault();
              $detailsLink[0].click();
            }
          });
          
          // Visual feedback on hover
          $logRow.css('cursor', 'pointer');
        });
        
        // Auto-refresh functionality (if enabled in settings)
        if (settings.nativeAiJs && settings.nativeAiJs.autoRefresh) {
          const refreshInterval = settings.nativeAiJs.refreshInterval || 60000; // Default: 1 minute
          
          // Set up periodic refresh
          setInterval(function() {
            Drupal.behaviors.nativeAiJsLogsDashboard.refreshDashboard($dashboard);
          }, refreshInterval);
        }
      });
    },
    
    /**
     * Initialize tooltip for operation bar.
     *
     * @param {jQuery} $element
     *   The element to initialize tooltip on.
     */
    initializeTooltip: function($element) {
      // Use Drupal's built-in tooltip system if available
      if (Drupal.tooltips) {
        Drupal.tooltips.add($element[0]);
      }
    },
    
    /**
     * Refresh dashboard data via AJAX.
     *
     * @param {jQuery} $dashboard
     *   The dashboard element to refresh.
     */
    refreshDashboard: function($dashboard) {
      const blockId = $dashboard.closest('.block').attr('id');
      
      if (!blockId) {
        return;
      }
      
      // Extract block uuid and view mode from the block ID
      const blockParts = blockId.match(/block-([a-f0-9-]+)/);
      
      if (!blockParts || blockParts.length < 2) {
        return;
      }
      
      const uuid = blockParts[1];
      
      // Perform AJAX request to refresh the block
      $.ajax({
        url: Drupal.url('native-aijs/ajax/refresh-logs-block'),
        type: 'GET',
        data: {
          'uuid': uuid
        },
        success: function(response) {
          if (response.content) {
            // Replace the dashboard content
            $dashboard.replaceWith(response.content);
            
            // Re-attach behaviors to the new content
            Drupal.attachBehaviors($dashboard.parent()[0]);
          }
        }
      });
    }
  };

})(jQuery, Drupal);