<?php

namespace Drupal\Tests\drupal_native_ai_nativeaijs\Unit\Plugin\Block;

use Drupal\Tests\UnitTestCase;
use Drupal\drupal_native_ai_nativeaijs\Plugin\Block\NativeAiJsLogsBlock;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Unit tests for the NativeAiJsLogsBlock.
 *
 * @group drupal_native_ai_nativeaijs
 * @coversDefaultClass \Drupal\drupal_native_ai_nativeaijs\Plugin\Block\NativeAiJsLogsBlock
 */
class NativeAiJsLogsBlockTest extends UnitTestCase {

  /**
   * The mocked database connection.
   *
   * @var \Drupal\Core\Database\Connection|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $database;

  /**
   * The mocked date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $dateFormatter;

  /**
   * The mocked current user.
   *
   * @var \Drupal\Core\Session\AccountInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $currentUser;

  /**
   * The mocked string translation service.
   *
   * @var \Drupal\Core\StringTranslation\TranslationInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $translation;

  /**
   * The block configuration.
   *
   * @var array
   */
  protected $configuration;

  /**
   * The block plugin ID.
   *
   * @var string
   */
  protected $pluginId;

  /**
   * The block plugin definition.
   *
   * @var array
   */
  protected $pluginDefinition;

  /**
   * The block plugin.
   *
   * @var \Drupal\drupal_native_ai_nativeaijs\Plugin\Block\NativeAiJsLogsBlock
   */
  protected $block;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Mock database connection.
    $this->database = $this->createMock(Connection::class);

    // Mock date formatter.
    $this->dateFormatter = $this->createMock(DateFormatterInterface::class);

    // Mock current user.
    $this->currentUser = $this->createMock(AccountInterface::class);

    // Mock string translation.
    $this->translation = $this->createMock(TranslationInterface::class);
    $this->translation->method('translate')
      ->willReturnCallback(function ($string) {
        return $string;
      });

    // Set up the block configuration.
    $this->configuration = [
      'log_entry_limit' => 5,
      'cache_time' => 300,
    ];
    $this->pluginId = 'native_aijs_logs_block';
    $this->pluginDefinition = [
      'provider' => 'drupal_native_ai_nativeaijs',
    ];

    // Create the block plugin instance.
    $this->block = new NativeAiJsLogsBlock(
      $this->configuration,
      $this->pluginId,
      $this->pluginDefinition,
      $this->database,
      $this->dateFormatter,
      $this->currentUser
    );

    // Set the string translation service.
    $container = $this->createMock(ContainerInterface::class);
    $container->method('get')
      ->with('string_translation')
      ->willReturn($this->translation);
    \Drupal::setContainer($container);

    // Set a method to handle translation.
    $translationMethod = function ($string, array $args = [], array $options = []) {
      return strtr($string, $args);
    };

    // Set the translation method on the block.
    $translateReflection = new \ReflectionClass($this->block);
    if ($translateReflection->hasMethod('t')) {
      $tMethod = $translateReflection->getMethod('t');
      $tMethod->setAccessible(TRUE);
      $tMethod->setClosure($this->block, $translationMethod);
    }
  }

  /**
   * Tests the block creation using the static create method.
   *
   * @covers ::create
   */
  public function testCreate() {
    $container = $this->createMock(ContainerInterface::class);
    $container->expects($this->exactly(3))
      ->method('get')
      ->willReturnMap([
        ['database', ContainerInterface::EXCEPTION_ON_INVALID_REFERENCE, $this->database],
        ['date.formatter', ContainerInterface::EXCEPTION_ON_INVALID_REFERENCE, $this->dateFormatter],
        ['current_user', ContainerInterface::EXCEPTION_ON_INVALID_REFERENCE, $this->currentUser],
      ]);

    $instance = NativeAiJsLogsBlock::create(
      $container,
      $this->configuration,
      $this->pluginId,
      $this->pluginDefinition
    );

    $this->assertInstanceOf(NativeAiJsLogsBlock::class, $instance);
  }

  /**
   * Tests the defaultConfiguration method.
   *
   * @covers ::defaultConfiguration
   */
  public function testDefaultConfiguration() {
    $expected = [
      'log_entry_limit' => 5,
      'cache_time' => 300,
    ];
    $this->assertEquals($expected, $this->block->defaultConfiguration());
  }

  /**
   * Tests the blockForm method.
   *
   * @covers ::blockForm
   */
  public function testBlockForm() {
    $form = [];
    $form_state = $this->createMock(FormStateInterface::class);
    
    $result = $this->block->blockForm($form, $form_state);
    
    $this->assertArrayHasKey('log_entry_limit', $result);
    $this->assertArrayHasKey('cache_time', $result);
    $this->assertEquals(5, $result['log_entry_limit']['#default_value']);
    $this->assertEquals(300, $result['cache_time']['#default_value']);
  }

  /**
   * Tests the blockSubmit method.
   *
   * @covers ::blockSubmit
   */
  public function testBlockSubmit() {
    $form = [];
    $form_state = $this->createMock(FormStateInterface::class);
    
    $form_state->expects($this->exactly(2))
      ->method('getValue')
      ->willReturnMap([
        ['log_entry_limit', 10],
        ['cache_time', 600],
      ]);
    
    $this->block->blockSubmit($form, $form_state);
    
    $config = $this->block->getConfiguration();
    $this->assertEquals(10, $config['log_entry_limit']);
    $this->assertEquals(600, $config['cache_time']);
  }

  /**
   * Tests the access control.
   *
   * @covers ::blockAccess
   */
  public function testBlockAccess() {
    $account = $this->createMock(AccountInterface::class);
    
    // Test with permission.
    $account->expects($this->once())
      ->method('hasPermission')
      ->with('view native ai logs')
      ->willReturn(TRUE);
    
    $access = $this->block->blockAccess($account);
    $this->assertEquals(AccessResult::allowed(), $access);
  }

  /**
   * Tests the build method.
   *
   * @covers ::build
   */
  public function testBuild() {
    // Mock the getLogStatistics method to return test data.
    $stats = [
      'total_calls' => 100,
      'avg_response_time' => 150.5,
      'error_rate' => 2.5,
      'most_used_model' => 'llama-2-7b',
      'operations' => [
        'text_generation' => 80,
        'embedding' => 20,
      ],
    ];
    
    // Mock the getRecentLogs method to return test data.
    $logs = [
      [
        'id' => 1,
        'timestamp' => 1622547600,
        'formatted_time' => '2021-06-01 12:00',
        'operation_type' => 'text_generation',
        'message' => 'Generated text successfully',
        'status' => 'success',
        'model' => 'llama-2-7b',
        'request_id' => 'req-123',
        'performance_metrics' => ['response_time' => 120],
        'log_link' => '<a href="/admin/reports/native-ai-logs/1">Details</a>',
      ],
    ];
    
    // Create a partial mock of the block to test the build method.
    $block = $this->getMockBuilder(NativeAiJsLogsBlock::class)
      ->setConstructorArgs([
        $this->configuration,
        $this->pluginId,
        $this->pluginDefinition,
        $this->database,
        $this->dateFormatter,
        $this->currentUser,
      ])
      ->onlyMethods(['getLogStatistics', 'getRecentLogs'])
      ->getMock();
    
    $block->expects($this->once())
      ->method('getLogStatistics')
      ->willReturn($stats);
    
    $block->expects($this->once())
      ->method('getRecentLogs')
      ->with(5)
      ->willReturn($logs);
    
    $build = $block->build();
    
    $this->assertEquals('native_aijs_logs_summary', $build['#theme']);
    $this->assertEquals($stats, $build['#stats']);
    $this->assertEquals($logs, $build['#logs']);
    $this->assertArrayHasKey('#view_all_link', $build);
    $this->assertArrayHasKey('#attached', $build);
    $this->assertArrayHasKey('#cache', $build);
    $this->assertEquals(['drupal_native_ai_nativeaijs/logs_dashboard'], $build['#attached']['library']);
    $this->assertEquals(['user.permissions'], $build['#cache']['contexts']);
    $this->assertEquals(['drupal_native_ai_nativeaijs:logs'], $build['#cache']['tags']);
    $this->assertEquals(300, $build['#cache']['max-age']);
  }

  /**
   * Tests the cache metadata methods.
   *
   * @covers ::getCacheContexts
   * @covers ::getCacheTags
   * @covers ::getCacheMaxAge
   */
  public function testCacheMetadata() {
    $this->assertEquals(['user.permissions'], $this->block->getCacheContexts());
    $this->assertEquals(['drupal_native_ai_nativeaijs:logs'], $this->block->getCacheTags());
    $this->assertEquals(300, $this->block->getCacheMaxAge());
  }

  /**
   * Tests the getLogStatistics method.
   *
   * @covers ::getLogStatistics
   */
  public function testGetLogStatistics() {
    // Create a reflection of the block to access protected methods.
    $reflection = new \ReflectionClass($this->block);
    $method = $reflection->getMethod('getLogStatistics');
    $method->setAccessible(TRUE);

    // Set up database query mocks for total calls
    $totalQueryMock = $this->getMockBuilder(\stdClass::class)
      ->addMethods(['execute', 'fetchField', 'condition', 'countQuery'])
      ->getMock();
    $totalQueryMock->expects($this->once())
      ->method('condition')
      ->willReturnSelf();
    $totalQueryMock->expects($this->once())
      ->method('countQuery')
      ->willReturnSelf();
    $totalQueryMock->expects($this->once())
      ->method('execute')
      ->willReturnSelf();
    $totalQueryMock->expects($this->once())
      ->method('fetchField')
      ->willReturn(100);

    // Set up database query mocks for errors
    $errorQueryMock = $this->getMockBuilder(\stdClass::class)
      ->addMethods(['execute', 'fetchField', 'condition', 'countQuery'])
      ->getMock();
    $errorQueryMock->expects($this->exactly(2))
      ->method('condition')
      ->willReturnSelf();
    $errorQueryMock->expects($this->once())
      ->method('countQuery')
      ->willReturnSelf();
    $errorQueryMock->expects($this->once())
      ->method('execute')
      ->willReturnSelf();
    $errorQueryMock->expects($this->once())
      ->method('fetchField')
      ->willReturn(5);

    // Set up database query mocks for average time
    $avgTimeQueryMock = $this->getMockBuilder(\stdClass::class)
      ->addMethods(['execute', 'fetchAssoc', 'condition', 'fields'])
      ->getMock();
    $avgTimeQueryMock->expects($this->once())
      ->method('condition')
      ->willReturnSelf();
    $avgTimeQueryMock->expects($this->once())
      ->method('fields')
      ->willReturnSelf();
    $avgTimeQueryMock->expects($this->once())
      ->method('execute')
      ->willReturnSelf();
    $avgTimeQueryMock->expects($this->exactly(2))
      ->method('fetchAssoc')
      ->willReturnOnConsecutiveCalls(
        ['performance_metrics' => serialize(['response_time' => 150])],
        ['performance_metrics' => serialize(['response_time' => 250])],
        FALSE
      );

    // Set up database query mocks for model usage
    $modelQueryMock = $this->getMockBuilder(\stdClass::class)
      ->addMethods(['execute', 'fetchAssoc', 'condition', 'fields', 'groupBy', 'addExpression', 'orderBy', 'range'])
      ->getMock();
    $modelQueryMock->expects($this->once())
      ->method('condition')
      ->willReturnSelf();
    $modelQueryMock->expects($this->once())
      ->method('fields')
      ->willReturnSelf();
    $modelQueryMock->expects($this->once())
      ->method('groupBy')
      ->willReturnSelf();
    $modelQueryMock->expects($this->once())
      ->method('addExpression')
      ->willReturnSelf();
    $modelQueryMock->expects($this->once())
      ->method('orderBy')
      ->willReturnSelf();
    $modelQueryMock->expects($this->once())
      ->method('range')
      ->willReturnSelf();
    $modelQueryMock->expects($this->once())
      ->method('execute')
      ->willReturnSelf();
    $modelQueryMock->expects($this->once())
      ->method('fetchAssoc')
      ->willReturn(['model' => 'llama-2-7b', 'count' => 80]);

    // Set up database query mocks for operation types
    $operationQueryMock = $this->getMockBuilder(\stdClass::class)
      ->addMethods(['execute', 'fetchAssoc', 'condition', 'fields', 'groupBy', 'addExpression', 'orderBy'])
      ->getMock();
    $operationQueryMock->expects($this->once())
      ->method('condition')
      ->willReturnSelf();
    $operationQueryMock->expects($this->once())
      ->method('fields')
      ->willReturnSelf();
    $operationQueryMock->expects($this->once())
      ->method('groupBy')
      ->willReturnSelf();
    $operationQueryMock->expects($this->once())
      ->method('addExpression')
      ->willReturnSelf();
    $operationQueryMock->expects($this->once())
      ->method('orderBy')
      ->willReturnSelf();
    $operationQueryMock->expects($this->once())
      ->method('execute')
      ->willReturnSelf();
    $operationQueryMock->expects($this->exactly(3))
      ->method('fetchAssoc')
      ->willReturnOnConsecutiveCalls(
        ['operation_type' => 'text_generation', 'count' => 80],
        ['operation_type' => 'embedding', 'count' => 20],
        FALSE
      );

    // Set up the database select method to return different mocks
    $this->database->expects($this->exactly(5))
      ->method('select')
      ->with('drupal_native_ai_nativeaijs_logs')
      ->willReturnOnConsecutiveCalls(
        $totalQueryMock,
        $errorQueryMock,
        $avgTimeQueryMock,
        $modelQueryMock,
        $operationQueryMock
      );

    // Call the protected method
    $stats = $method->invoke($this->block);

    // Verify the results
    $this->assertEquals(100, $stats['total_calls']);
    $this->assertEquals(5.0, $stats['error_rate']);
    $this->assertEquals(200.0, $stats['avg_response_time']);
    $this->assertEquals('llama-2-7b', $stats['most_used_model']);
    $this->assertEquals(['text_generation' => 80, 'embedding' => 20], $stats['operations']);
  }

  /**
   * Tests the getRecentLogs method.
   *
   * @covers ::getRecentLogs
   */
  public function testGetRecentLogs() {
    // Create a reflection of the block to access protected methods.
    $reflection = new \ReflectionClass($this->block);
    $method = $reflection->getMethod('getRecentLogs');
    $method->setAccessible(TRUE);

    // Set up database query mocks
    $queryMock = $this->getMockBuilder(\stdClass::class)
      ->addMethods(['execute', 'fetchAssoc', 'fields', 'orderBy', 'range'])
      ->getMock();
    $queryMock->expects($this->once())
      ->method('fields')
      ->willReturnSelf();
    $queryMock->expects($this->once())
      ->method('orderBy')
      ->willReturnSelf();
    $queryMock->expects($this->once())
      ->method('range')
      ->willReturnSelf();
    $queryMock->expects($this->once())
      ->method('execute')
      ->willReturnSelf();
    $queryMock->expects($this->exactly(2))
      ->method('fetchAssoc')
      ->willReturnOnConsecutiveCalls(
        [
          'id' => 1,
          'timestamp' => 1622547600,
          'operation_type' => 'text_generation',
          'message' => 'Generated text successfully',
          'status' => 'success',
          'model' => 'llama-2-7b',
          'request_id' => 'req-123',
          'performance_metrics' => serialize(['response_time' => 120]),
        ],
        FALSE
      );

    // Set up the database select method
    $this->database->expects($this->once())
      ->method('select')
      ->with('drupal_native_ai_nativeaijs_logs')
      ->willReturn($queryMock);

    // Set up the date formatter
    $this->dateFormatter->expects($this->once())
      ->method('format')
      ->with(1622547600, 'short')
      ->willReturn('2021-06-01 12:00');

    // Call the protected method
    $logs = $method->invoke($this->block, 5);

    // Verify the results
    $this->assertCount(1, $logs);
    $this->assertEquals(1, $logs[0]['id']);
    $this->assertEquals('2021-06-01 12:00', $logs[0]['formatted_time']);
    $this->assertEquals('text_generation', $logs[0]['operation_type']);
    $this->assertEquals('success', $logs[0]['status']);
    $this->assertEquals('llama-2-7b', $logs[0]['model']);
    $this->assertEquals(['response_time' => 120], $logs[0]['performance_metrics']);
    $this->assertArrayHasKey('log_link', $logs[0]);
  }

}