---
id: migration-DEPRECATION_MIGRATION_CHECKLIST
---
# Deprecation Migration Checklist for Module Maintainers

## Pre-Migration Checklist

### 1. Impact Assessment
- [ ] Identify all modules that depend on your deprecated component
- [ ] List all services that will be affected
- [ ] Document all API endpoints that will change
- [ ] Review configuration schemas that need updates
- [ ] Check for hardcoded references in custom code

### 2. Communication Planning
- [ ] Notify all stakeholders 6 months before deprecation
- [ ] Create migration guide documentation
- [ ] Set up support channel for migration questions
- [ ] Schedule migration assistance sessions
- [ ] Prepare FAQ document

## LangChain Module Migration Checklist

### Phase 1: Preparation (Before July 2025)
- [ ] Export all LangChain configurations
  ```bash
  drush config:export --destination=/tmp/langchain-export
  ```
- [ ] Backup LangChain entities
  ```bash
  drush sql-query "SELECT * FROM langchain_chain" > /tmp/langchain_chains.sql
  drush sql-query "SELECT * FROM langchain_agent" > /tmp/langchain_agents.sql
  ```
- [ ] Document custom chains and workflows
- [ ] Identify all modules using LangChain services

### Phase 2: Code Updates (July 2025)
- [ ] Update service references:
  - [ ] Replace `langchain.service` with `llm.workflow_manager`
  - [ ] Replace `langchain.provider_manager` with `drupal_native_ai.provider_manager`
  - [ ] Replace `langchain.mcp_integration` with `mcp_client_extras.agent_fleet_orchestrator`
- [ ] Update module dependencies:
  - [ ] Remove from `llm_platform_recipe.recipe.yml`
  - [ ] Update `drupal_native_ai` submodule
  - [ ] Remove from all `.info.yml` optional_dependencies
- [ ] Migrate configurations:
  ```php
  // Old configuration
  $config['langchain.settings']['default_provider'] = 'openai';
  
  // New configuration
  $config['drupal_native_ai.settings']['default_provider'] = 'openai';
  ```

### Phase 3: Testing (August 2025)
- [ ] Run migration scripts
- [ ] Test all workflows that used LangChain
- [ ] Verify API endpoint migrations
- [ ] Check performance metrics
- [ ] Validate data integrity

### Phase 4: Cleanup (September 2025)
- [ ] Uninstall LangChain modules
- [ ] Remove LangChain code from repository
- [ ] Archive to `__DELETE_LATER/langchain_2025-09-30/`
- [ ] Update documentation
- [ ] Remove from CI/CD pipelines

## AI Swarm Intelligence Submodules Migration

### code_genesis Migration
- [ ] Update all references from:
  ```php
  \Drupal::service('code_genesis.engine')
  ```
  To:
  ```php
  \Drupal::service('ai_swarm_intelligence.orchestrator')->createCodeGenerationSwarm()
  ```
- [ ] Migrate configuration settings
- [ ] Update ECA models using code_genesis
- [ ] Test code generation functionality
- [ ] Archive module to `__DELETE_LATER/code_genesis_2025-07-31/`

### project_intelligence Migration
- [ ] Update service calls from:
  ```php
  \Drupal::service('project_intelligence.facade')
  ```
  To:
  ```php
  \Drupal::service('ai_swarm_intelligence.orchestrator')->createProjectIntelligenceSwarm()
  ```
- [ ] Migrate project analysis configurations
- [ ] Update dependent workflows
- [ ] Verify analysis capabilities
- [ ] Archive module to `__DELETE_LATER/project_intelligence_2025-07-31/`

## Standalone Test Files Migration

### Test Structure Migration
- [ ] Identify all standalone test files:
  ```bash
  find . -name "*test*.php" -not -path "*/tests/*" -not -path "*/vendor/*"
  ```
- [ ] For each test file:
  - [ ] Determine test type (Unit/Kernel/Functional)
  - [ ] Create proper PHPUnit test class
  - [ ] Move to appropriate directory
  - [ ] Update namespace and use statements
  - [ ] Convert assertions to PHPUnit format

### Example Conversion
```php
// Old standalone test
class MyTestRunner {
    public function testFeature() {
        $result = some_function();
        if ($result !== 'expected') {
            throw new Exception('Test failed');
        }
    }
}

// New PHPUnit test
namespace Drupal\Tests\my_module\Unit;

use Drupal\Tests\UnitTestCase;

class MyFeatureTest extends UnitTestCase {
    public function testFeature() {
        $result = some_function();
        $this->assertEquals('expected', $result);
    }
}
```

### CI/CD Updates
- [ ] Update test commands in `.gitlab-ci.yml`
- [ ] Remove references to standalone test runners
- [ ] Update code coverage configuration
- [ ] Verify all tests run in pipeline

## Service Consolidation Checklist

### Service Inventory
- [ ] List all services in your module
- [ ] Identify duplicate functionality
- [ ] Map service dependencies
- [ ] Document service consumers

### Consolidation Steps
- [ ] Create service aliases for backward compatibility
- [ ] Update service definitions
- [ ] Test service injection
- [ ] Remove duplicate implementations
- [ ] Update documentation

## Configuration Migration Checklist

### Legacy Configuration
- [ ] Identify deprecated configuration keys
- [ ] Create configuration update hooks
- [ ] Implement backward compatibility
- [ ] Test configuration upgrades
- [ ] Document configuration changes

### Update Hook Example
```php
/**
 * Migrate LangChain configuration to native AI settings.
 */
function my_module_update_9001() {
  $config = \Drupal::configFactory()->getEditable('langchain.settings');
  $old_provider = $config->get('default_provider');
  
  if ($old_provider) {
    \Drupal::configFactory()
      ->getEditable('drupal_native_ai.settings')
      ->set('default_provider', $old_provider)
      ->save();
    
    $config->delete();
  }
}
```

## Post-Migration Verification

### Functionality Testing
- [ ] All features work as expected
- [ ] No regression in functionality
- [ ] Performance meets or exceeds previous levels
- [ ] Error logs are clean

### Documentation Updates
- [ ] README files updated
- [ ] API documentation reflects changes
- [ ] Migration guide published
- [ ] Change log updated

### Monitoring Setup
- [ ] Set up alerts for deprecated service usage
- [ ] Monitor error rates during migration
- [ ] Track performance metrics
- [ ] Collect user feedback

## Rollback Procedures

### Preparation
- [ ] Create full backup before migration
- [ ] Document rollback steps
- [ ] Test rollback procedure
- [ ] Set up feature flags for gradual rollout

### Rollback Steps
1. [ ] Restore database backup
2. [ ] Revert code changes
3. [ ] Clear caches
4. [ ] Verify functionality
5. [ ] Communicate rollback to stakeholders

## Support Resources

### Documentation
- Full deprecation timeline: [DEPRECATION_SUNSET_TIMELINE.md](./DEPRECATION_SUNSET_TIMELINE)
- Visual timeline: [DEPRECATION_VISUAL_TIMELINE.md](./DEPRECATION_VISUAL_TIMELINE)
- Migration guides: `/docs/migration/`

### Getting Help
- Slack channel: #module-deprecation-support
- Email: deprecation-support@bluefly.io
- Office hours: Wednesdays 2-4 PM EST

### Automated Tools
```bash
# Check for deprecated service usage
drush deprecation:check my_module

# Run migration assistant
drush migrate:deprecated --module=my_module

# Validate migration completeness
drush migrate:validate my_module
```

---

**Remember**: Always test migrations in a development environment first!
