---
id: migration-DEPRECATION_SUNSET_TIMELINE
---
# Comprehensive Deprecation and Sunset Timeline for Drupal Modules

## Executive Summary

This document provides a complete deprecation and sunset timeline for all obsolete components in the Drupal modules ecosystem. Components are organized by deprecation urgency, dependency impact, and migration complexity.

## Timeline Overview

```
Q3 2025 (Immediate)     Q4 2025 (Short-term)     Q1 2026 (Medium-term)     Q2 2026+ (Long-term)
───────────────────────┼────────────────────────┼──────────────────────────┼─────────────────────
Critical Deprecations  │ Module Retirements     │ Service Migrations       │ Final Transitions
```

## Phase 1: Immediate Deprecations (Q3 2025)

### 1.1 LangChain Module Suite
**Target Date: July 15, 2025**
**Status**: DEPRECATED - Critical Priority

#### Components to Retire:
- `langchain` (main module)
- `langchain_ai` 
- `langchain_eca`
- `langchain_rag`
- `langchain_langgraph`
- `langfuse_observability`
- `llm_adapter`

#### Migration Actions Required:
1. **Week 1-2**: Export all LangChain configurations
   ```bash
   drush config:export --destination=/tmp/langchain-export
   drush sql-query "SELECT * FROM langchain_chain" > /tmp/langchain_chains.sql
   drush sql-query "SELECT * FROM langchain_agent" > /tmp/langchain_agents.sql
   ```

2. **Week 3-4**: Update dependent modules
   - Remove from `llm_platform_recipe.recipe.yml` (line 121)
   - Update `drupal_native_ai` to remove langchain integration
   - Remove from all optional_dependencies in `.info.yml` files

3. **Week 5-6**: Migrate services
   - `langchain.service` → `llm.workflow_manager`
   - `langchain.provider_manager` → `drupal_native_ai.provider_manager`
   - `langchain.mcp_integration` → `mcp_client_extras.agent_fleet_orchestrator`

4. **Week 7-8**: Complete uninstallation
   ```bash
   drush pm:uninstall langchain_rag langchain_langgraph langfuse_observability langchain_eca langchain_ai
   drush pm:uninstall langchain
   ```

**Milestone**: Archive to `__DELETE_LATER/langchain_2025-07-15/`

### 1.2 AI Swarm Intelligence Deprecated Submodules
**Target Date: July 31, 2025**
**Status**: DEPRECATED - High Priority

#### Components to Retire:
- `ai_swarm_intelligence/modules/code_genesis/`
- `ai_swarm_intelligence/modules/project_intelligence/`

#### Migration Actions Required:
1. **Week 1**: Document migration paths
   - Update all references to use `SwarmOrchestrator->createCodeGenerationSwarm()`
   - Replace `ProjectIntelligenceFacade` with `SwarmOrchestrator->createProjectIntelligenceSwarm()`

2. **Week 2**: Update dependent code
   - Search and replace all service references
   - Update any ECA models using these services

3. **Week 3**: Test migration completeness
   - Verify all functionality available through main module
   - Run full test suite

**Milestone**: Archive to `__DELETE_LATER/ai_swarm_deprecated_modules_2025-07-31/`

### 1.3 Standalone Test Files
**Target Date: August 15, 2025**
**Status**: Technical Debt - High Priority

#### Components to Migrate:
- All standalone PHP test runners in module root directories
- Shell script test runners
- Debug files

#### Migration Actions Required:
1. **Week 1-2**: Convert to PHPUnit format
   - Move unit tests to `tests/src/Unit/`
   - Move kernel tests to `tests/src/Kernel/`
   - Move functional tests to `tests/src/Functional/`

2. **Week 3**: Update CI/CD pipelines
   - Remove references to standalone test runners
   - Update test execution commands

**Milestone**: Archive to `__DELETE_LATER/test_runners_2025-08-15/`

## Phase 2: Short-term Retirements (Q4 2025)

### 2.1 Legacy Service Implementations
**Target Date: October 15, 2025**
**Status**: Service Consolidation

#### Components to Retire:
- Duplicate service implementations across modules
- Legacy API endpoints
- Deprecated event subscribers

#### Migration Actions Required:
1. **September**: Service inventory and mapping
   - Document all duplicate services
   - Create service migration map
   - Identify consumers of deprecated services

2. **October**: Service consolidation
   - Implement service aliases for backward compatibility
   - Update all service references
   - Remove duplicate implementations

**Milestone**: Consolidated service architecture

### 2.2 Alternative Services Module
**Target Date: November 30, 2025**
**Status**: Evaluate for Deprecation

#### Evaluation Criteria:
- If functionality fully replaced by core services
- If no active usage in production
- If maintenance burden exceeds value

#### Conditional Actions:
1. **If deprecating**:
   - Migrate proxy service functionality to core
   - Update DDEV integration approach
   - Archive service discovery patterns

2. **If keeping**:
   - Modernize codebase
   - Remove legacy components
   - Update documentation

**Milestone**: Decision checkpoint with go/no-go

## Phase 3: Medium-term Migrations (Q1-Q2 2026)

### 3.1 Module Consolidations
**Target Date: January-March 2026**
**Status**: Architecture Optimization

#### Consolidation Targets:
1. **API Normalization Submodules**
   - Merge related submodules into core
   - Consolidate service definitions
   - Unify configuration schemas

2. **LLM Module Submodules**
   - Evaluate overlapping functionality
   - Consolidate provider integrations
   - Merge insights modules

3. **Security Module Suite**
   - Consolidate compliance frameworks
   - Merge overlapping security controls
   - Unify audit logging

#### Migration Actions:
1. **Q1 2026**: Planning and dependency analysis
2. **Q2 2026**: Phased consolidation with feature flags
3. **Q3 2026**: Complete migration and cleanup

**Milestone**: Simplified module architecture

### 3.2 Legacy Configuration Formats
**Target Date: April 2026**
**Status**: Configuration Modernization

#### Components to Migrate:
- YAML configuration to config entities
- Legacy schema definitions
- Deprecated configuration keys

#### Migration Path:
1. **Month 1**: Create configuration upgrade paths
2. **Month 2**: Implement backward compatibility layer
3. **Month 3**: Complete migration with update hooks

**Milestone**: Modern configuration management

## Phase 4: Long-term Transitions (Beyond Q2 2026)

### 4.1 Technology Stack Updates
**Target Date: Q3 2026+**
**Status**: Future Planning

#### Potential Transitions:
1. **WebSocket to Server-Sent Events**
   - Evaluate modern real-time alternatives
   - Plan migration for real-time features

2. **Custom Entity Types to Core Solutions**
   - Evaluate Drupal core improvements
   - Migrate to standard solutions where possible

3. **Legacy Integration Patterns**
   - Update to modern API standards
   - Implement GraphQL where appropriate

### 4.2 Architecture Evolution
**Target Date: 2027+**
**Status**: Strategic Planning

#### Evolution Areas:
1. **Microservices Decomposition**
   - Extract heavy processing to services
   - Implement event-driven architecture

2. **AI Model Standardization**
   - Converge on standard AI interfaces
   - Implement model-agnostic patterns

## Dependency Considerations

### Critical Dependencies Matrix

| Module | Depends On | Migration Complexity | Priority |
|--------|------------|---------------------|----------|
| langchain | None | High (many dependents) | Critical |
| code_genesis | ai_swarm_intelligence | Medium | High |
| project_intelligence | ai_swarm_intelligence | Medium | High |
| alternative_services | Core services | Low | Medium |
| Legacy test files | PHPUnit | Low | High |

### Update Order Requirements

1. **First Wave** (No dependencies):
   - Standalone test files
   - Debug scripts
   - Shell scripts

2. **Second Wave** (Leaf modules):
   - langchain submodules
   - Deprecated AI Swarm submodules

3. **Third Wave** (Core modules):
   - langchain main module
   - Service consolidations

4. **Fourth Wave** (Architecture changes):
   - Module consolidations
   - Configuration migrations

## Risk Mitigation Strategies

### 1. Feature Flags
```php
// Enable gradual rollout
if (\Drupal::config('system.features')->get('use_legacy_langchain')) {
  // Legacy code path
} else {
  // New implementation
}
```

### 2. Compatibility Layers
```php
// Service aliases for backward compatibility
services:
  langchain.service:
    alias: llm.workflow_manager
    deprecated: 'The "%alias_id%" service is deprecated. Use "llm.workflow_manager" instead.'
```

### 3. Automated Migration Tools
```bash
# Automated configuration migration
drush migrate:langchain-to-native --dry-run
drush migrate:langchain-to-native --execute
```

## Communication Plan

### Stakeholder Notifications

1. **6 Months Before** (January 2025):
   - Announce deprecation plans
   - Publish migration guides
   - Open feedback period

2. **3 Months Before** (April 2025):
   - Final deprecation notice
   - Migration tool availability
   - Support channel setup

3. **1 Month Before** (June 2025):
   - Last call for migrations
   - Final compatibility check
   - Cutover planning

### Documentation Updates

1. **Migration Guides**: Complete guides for each deprecated component
2. **API Documentation**: Updated with deprecation notices
3. **Change Logs**: Detailed deprecation entries
4. **Support Resources**: FAQs and troubleshooting guides

## Success Metrics

### Phase 1 Success Criteria
- [ ] 100% of LangChain functionality migrated
- [ ] All dependent modules updated
- [ ] Zero regression in test coverage
- [ ] Migration tools tested and documented

### Phase 2 Success Criteria
- [ ] Service consolidation reduces duplicates by 50%
- [ ] API response time improved by 20%
- [ ] Maintenance burden reduced by 30%

### Phase 3 Success Criteria
- [ ] Module count reduced by 25%
- [ ] Configuration complexity reduced
- [ ] Developer onboarding time decreased

### Phase 4 Success Criteria
- [ ] Architecture modernization complete
- [ ] Technology stack current
- [ ] Future-proof design patterns

## Monitoring and Rollback

### Monitoring Points
1. **Pre-deprecation**: Usage metrics baseline
2. **During migration**: Error rates and performance
3. **Post-deprecation**: System stability metrics

### Rollback Procedures
1. **Feature flags**: Instant rollback capability
2. **Database backups**: Point-in-time recovery
3. **Code archives**: Full version history in `__DELETE_LATER`

## Conclusion

This deprecation and sunset timeline provides a structured approach to modernizing the Drupal modules ecosystem while minimizing disruption. By following this timeline, we can:

1. Remove technical debt systematically
2. Improve system performance and maintainability
3. Simplify the developer experience
4. Prepare for future architectural improvements

Regular reviews and adjustments to this timeline will ensure we stay responsive to changing requirements while maintaining system stability.

---

**Document Version**: 1.0  
**Last Updated**: January 2025  
**Next Review**: March 2025  
**Owner**: Bluefly AI Platform Team
