---
id: modules-development-native_ai-CONTRIBUTING
---
# Contributing to Drupal Native AI

## Development Standards

### Code Quality and Testing

We use standardized tags for all actionable comments in the codebase:

- `TODO`: General technical debt or future work
- `TODO-TEST`: Test coverage required for a method/class/feature
- `PERFORMANCE-TEST`: Performance test required
- `ROADMAP-TEST`: Test linked to a roadmap requirement
- `NOTE-COVERAGE`: Special coverage or edge case note

#### Test Annotation Format
```php
/**
 * TODO-TEST: [Test name - use method name + scenario being tested]
 *
 * [Description of what to test including specific scenarios, edge cases, and expected behavior]
 *
 * Test should verify:
 * - [Key validation point 1]
 * - [Key validation point 2]
 *
 * @covers \Drupal\[module]\[Namespace]\[Class]::[method]
 * @group [module_name]
 * @group tdd
 * @see ROADMAP.md (Section: [specific_section] - [specific_task])
 * @priority [high|medium|low]
 * @complexity [high|medium|low]
 * @testType [Unit|Kernel|Functional|FunctionalJavascript]
 * @issue [optional issue/ticket reference]
 */
```

### Development Environment

#### IDE Configuration

We recommend using the `todo-tree` extension for VSCode or Cursor. Add the following to your `.vscode/settings.json`:

```json
{
  "todo-tree.highlights.useColourScheme": true,
  "todo-tree.general.tags": [
    "TODO", "TODO-TEST", "PERFORMANCE-TEST", "ROADMAP-TEST", "NOTE-COVERAGE"
  ],
  "todo-tree.highlights.customHighlight": {
    "TODO": { "foreground": "#fff", "background": "#e57373", "icon": "alert" },
    "TODO-TEST": { "foreground": "#fff", "background": "#64b5f6", "icon": "beaker" },
    "PERFORMANCE-TEST": { "foreground": "#fff", "background": "#ffd54f", "icon": "dashboard" },
    "ROADMAP-TEST": { "foreground": "#fff", "background": "#81c784", "icon": "roadmap" }
  }
}
```

### Development Workflow

1. Fork the specific module repository
2. Create a feature branch
3. Implement changes
4. Add tests
5. Update documentation
6. Submit a pull request

### Quality Assurance

#### Automated Testing
- Unit tests (PHPUnit)
- Kernel tests
- Functional tests
- JavaScript tests
- Accessibility tests

#### Code Quality
- PHP_CodeSniffer (Drupal standards)
- PHPStan (static analysis)
- ESLint (JavaScript)
- Stylelint (CSS)

#### Documentation
- README.md
- API documentation
- Usage examples
- Change records
- Release notes

### CI/CD Integration

- The CI pipeline will fail if any `@priority high` TODOs remain before a production release
- TODO summaries are published as pipeline artifacts for team review
- Automated testing runs on every pull request
- Code quality checks are enforced
- Documentation is automatically built and published

### TODO Management

Use the `bfcli todos` command to manage and track TODOs:

```bash
# Extract and summarize all TODOs
bfcli todos

# Generate TODO report in specific format
bfcli todos --format=markdown
bfcli todos --format=json
bfcli todos --format=html

# Filter TODOs by priority
bfcli todos --priority=high
```

## Module Contribution

### 1. Module Structure
```
module_name/
├── composer.json           # Module dependencies
├── module_name.info.yml    # Module metadata
├── module_name.module      # Hook implementations
├── src/                   # PHP classes
├── tests/                # Test suites
├── config/              # Configuration
└── docs/               # Documentation
```

### 2. Module Metadata
```yaml
# module_name.info.yml
name: Module Name
type: module
description: 'Module description.'
package: 'Custom'
core_version_requirement: ^11.0
dependencies:
  - drupal:system
  - drupal:user
```

### 3. Testing Requirements
- Unit tests for all services
- Integration tests for model execution
- Performance benchmarks
- Security scanning
- Accessibility testing

## Recipe Contribution

### 1. Recipe Structure
```yaml
# recipes/custom/recipe_name/recipe_name.recipe.yml
name: Recipe Name
type: recipe
description: 'Recipe description'
install:
  - module_name
  - other_module
config:
  import:
    - module_name.settings
```

### 2. Recipe Requirements
- Use semantic versioning
- Include dependencies
- Document requirements
- Provide validation
- Include rollback procedures

## Template Contribution

### 1. Template Structure
```
template_name/
├── template.info.yml
├── template.recipe.yml
├── config/
├── content/
└── docs/
```

### 2. Template Requirements
- Clear documentation
- Version compatibility
- Security compliance
- Performance benchmarks
- Accessibility standards

## Support

- [Issue Queue](https://gitlab.bluefly.io/llm/drupal/drupal_native_ai/-/issues)
- [Security Policy](security)
- [Community Guidelines](community) 
