---
id: modules-development-native_ai-INTEGRATION_PATTERN
---
# Drupal Native AI Integration Pattern

This document describes the integration pattern for making Drupal Native AI the default AI provider for all modules in the Drupal AI ecosystem.

## Overview

The Drupal Native AI module provides AI capabilities using Drupal's native features, pattern recognition, and statistical analysis without requiring external AI services. It can serve as the default AI provider for all modules in the Drupal AI ecosystem while still allowing commercial providers to be used when needed.

## Integration Architecture

The integration architecture follows the adapter pattern, which allows Drupal Native AI to seamlessly integrate with other modules:

1. **Base Adapter Class**: `NativeAiAdapterBase` provides the foundation for all adapters.
2. **Module-Specific Adapters**: Each module has its own adapter that extends the base adapter and implements the module's specific AI provider interface.
3. **Configurable Fallback**: Each operation can be configured to use a specific fallback provider if the native provider fails or isn't suitable.
4. **Layered Provider Architecture**: Providers are organized in a layered architecture with the native provider at the top and commercial providers as optional add-ons.

```
                     ┌─────────────────────┐
                     │ Drupal Native AI    │
                     │ (Default Provider)  │
                     └──────────┬──────────┘
                                │
                                ▼
                     ┌─────────────────────┐
                     │ NativeAiAdapterBase │
                     └──────────┬──────────┘
                                │
                    ┌───────────┴───────────┐
                    │                       │
                    ▼                       ▼
        ┌─────────────────────┐   ┌─────────────────────┐
        │ LlmModuleAdapter    │   │ SwarmIntelligence   │
        │                     │   │ Adapter             │
        └─────────────────────┘   └─────────────────────┘
                    │                       │
                    ▼                       ▼
        ┌─────────────────────┐   ┌─────────────────────┐
        │ Commercial Provider │   │ Commercial Provider │
        │ Plugins (Optional)  │   │ Plugins (Optional)  │
        └─────────────────────┘   └─────────────────────┘
```

## Adapter Implementation

### Base Adapter Class

The `NativeAiAdapterBase` class provides common functionality for all adapters:

- Configuration management
- Provider selection logic
- Fallback provider handling
- Operation logging and metrics
- Error handling

```php
abstract class NativeAiAdapterBase {
  protected function shouldUseNativeProvider(string $operation, array $context = []): bool;
  protected function getFallbackProvider(string $operation, array $context = []): ?object;
  abstract protected function loadFallbackProviderService(string $provider_id): ?object;
  protected function logOperation(string $operation, array $context, bool $success, string $provider, ?string $message = NULL): void;
}
```

### Module-Specific Adapters

Each module has a dedicated adapter that implements the module's specific AI provider interface:

1. **LLM Module Adapter**: Implements `LlmProviderInterface` from the LLM module.
2. **AI Swarm Intelligence Adapter**: Integrates with the AI Swarm Intelligence module.
3. **Langchain Adapter**: Integrates with the Langchain module.
4. **Field ECA Bridge Adapter**: Integrates with the Field ECA Bridge module.

## Commercial Provider Plugins

Commercial providers are implemented as submodules of the Drupal Native AI module:

1. **drupal_native_ai_openai**: OpenAI integration
2. **drupal_native_ai_anthropic**: Anthropic Claude integration
3. **drupal_native_ai_huggingface**: HuggingFace integration
4. **drupal_native_ai_ollama**: Ollama local deployment integration

Each provider module:

- Implements `AIProviderInterface`
- Registers itself with the native AI provider service
- Has its own configuration settings
- Provides model-specific capabilities and optimizations

## Configuration System

The configuration system allows users to:

1. **Force Native Provider**: Enable/disable forced usage of the native provider for all operations.
2. **Default Provider**: Select the default provider when force native is disabled.
3. **Operation-Specific Settings**: Configure which provider to use for specific operations.
4. **Fallback Providers**: Configure fallback providers for each operation.
5. **Provider Priorities**: Set the priority for each provider (higher priority providers are used first).
6. **Module Integrations**: Enable/disable integration with specific modules.

## Integration Example: LLM Module

Here's an example of how the LLM module integration works:

1. The LLM module tries to load an AI provider for text generation.
2. The `LlmModuleAdapter` is registered as a provider with the LLM module.
3. When the LLM module calls `generate()` on the adapter:
   - The adapter checks if it should use the native provider based on configuration.
   - If yes, it uses the native provider to generate a response using pattern analysis.
   - If no, or if the native provider fails, it falls back to the configured fallback provider.
4. The operation is logged, and metrics are collected for analysis.

```php
// Example of LLM module integration
public function generate(string $prompt, array $options = []): string {
  $operation = 'generate';
  $context = [
    'prompt' => $prompt,
    'options' => $options,
  ];

  if ($this->shouldUseNativeProvider($operation, $context)) {
    try {
      // Use the native AI provider...
      $content_data = [
        'type' => 'prompt',
        'content' => $prompt,
        'options' => $options,
      ];
      $analysis = $this->nativeAiProvider->analyzeContentPatterns($content_data);
      $response = $this->convertAnalysisToText($analysis);
      
      $this->logOperation($operation, $context, TRUE, 'drupal_native_ai');
      return $response;
    }
    catch (\Exception $e) {
      $this->logOperation($operation, $context, FALSE, 'drupal_native_ai', $e->getMessage());
      
      // Try fallback provider if available...
      $fallback = $this->getFallbackProvider($operation, $context);
      if ($fallback instanceof LlmProviderInterface) {
        return $fallback->generate($prompt, $options);
      }
      
      // Return a basic response if no fallback is available...
      return "I processed your prompt about " . substr($prompt, 0, 50) . "...";
    }
  }
  else {
    // Use configured fallback provider...
    $fallback = $this->getFallbackProvider($operation, $context);
    if ($fallback instanceof LlmProviderInterface) {
      return $fallback->generate($prompt, $options);
    }
    
    // Return a basic response if no fallback is available...
    return "I processed your prompt about " . substr($prompt, 0, 50) . "...";
  }
}
```

## Adding Support for a New Module

To add Drupal Native AI support to a new module:

1. Create a new adapter class that extends `NativeAiAdapterBase` and implements the module's AI provider interface.
2. Implement the required methods from the interface, routing operations through the native AI provider when appropriate.
3. Add a service definition in `drupal_native_ai.services.yml` that registers the adapter with the module.
4. Add configuration options in `drupal_native_ai.settings.yml` for the new module integration.
5. Update the settings form to include the new module integration options.

## Best Practices

1. **Progressive Enhancement**: Always implement the native provider as a progressive enhancement that can fall back to other providers.
2. **Clear Error Handling**: Provide clear error handling and logging for when operations fail.
3. **Configurable Fallbacks**: Allow users to configure fallback providers for each operation.
4. **Performance Monitoring**: Implement metrics collection to track performance and success rates.
5. **Conservative Defaults**: Default to using the native provider only for operations it can reliably handle.

## Benefits of This Approach

1. **Reduced Dependency**: Less reliance on external AI services for basic functionality.
2. **Privacy Protection**: Keeps user data within the Drupal ecosystem when possible.
3. **Cost Efficiency**: Reduces API calls to commercial providers for basic operations.
4. **Consistent Interface**: Provides a consistent interface for all AI operations.
5. **Graceful Degradation**: Allows for graceful degradation when external services are unavailable.
6. **Flexible Configuration**: Gives users control over which provider to use for each operation.

## Limitations

1. **Complexity Limitations**: The native provider can't match the capabilities of large language models for complex operations.
2. **Pattern-Based Approach**: The native approach is pattern-based rather than neural-network-based, which limits its generative capabilities.
3. **No True Embeddings**: The native provider generates pseudo-embeddings rather than true semantic embeddings.
4. **Limited Image and Audio Processing**: Image and audio processing capabilities are limited or non-existent without commercial providers.

## Future Enhancements

1. **Improved Pattern Recognition**: Enhance the pattern recognition capabilities of the native provider.
2. **Machine Learning Integration**: Integrate with simple machine learning libraries that can run locally.
3. **Caching and Optimization**: Implement intelligent caching and optimization for AI operations.
4. **Collaborative Intelligence**: Implement collaborative intelligence between the native provider and commercial providers.
5. **Federated Learning**: Explore federated learning approaches to improve the native provider without sharing user data.
