---
id: modules-development-native_ai-LLM_PLATFORM_INTEGRATION
---
# LLM Platform Recipe Integration

## Overview

We have successfully integrated the Drupal Native AI Agent Field System with the LLM Platform Recipe. This integration enables content types within the LLM Platform to leverage our agent field types, providing seamless AI capabilities in content creation and management workflows.

## Integration Components

### 1. AIHooks Class

Created a new `AIHooks` class in the LLM Platform Recipe that implements the integration with Drupal Native AI fields:

```php
namespace Drupal\llm_platform_recipe\RecipeOnboarding;

class AIHooks implements ContainerInjectionInterface {
  // ...
  
  public static function registerIntegrations() {
    return [
      'field_types' => [
        'agent_reference' => [
          'id' => 'drupal_native_ai_agent_reference',
          // ...
        ],
      ],
      'field_widgets' => [
        'agent_selection' => [
          'id' => 'drupal_native_ai_agent_selection',
          // ...
        ],
      ],
      'field_formatters' => [
        'agent_execution' => [
          'id' => 'drupal_native_ai_agent_execution',
          // ...
        ],
        'agent_reference' => [
          'id' => 'drupal_native_ai_agent_reference',
          // ...
        ],
      ],
      // ...
    ];
  }
}
```

This hook implementation registers our agent field types, widgets, and formatters with the LLM Platform Recipe system.

### 2. NativeAiBridge Updates

Enhanced the `NativeAiBridge` service to support the `AgentReferenceItem` field type:

- Added support for extracting agent IDs from agent reference fields
- Implemented agent execution for agent reference fields
- Added result handling and storage in agent reference fields
- Integrated with the Agent Registry for retrieving agent instances

```php
namespace Drupal\llm_platform_recipe\Service\ExternalService;

class NativeAiBridge {
  // ...
  
  protected function extractFieldContent(FieldDefinitionInterface $field_definition, $field_value) {
    // Added support for drupal_native_ai_agent_reference field type
    // ...
  }
  
  protected function updateFieldValue(FieldDefinitionInterface $field_definition, $field_value, $processed_content) {
    // Added support for drupal_native_ai_agent_reference field type
    // ...
  }
  
  protected function processWithAgent(FieldDefinitionInterface $field_definition, $field_value, array $options = []) {
    // Specialized processing for agent reference fields
    // ...
  }
}
```

### 3. Agent Field Integration Recipe

Created a new recipe file specifically for agent field integration:

```yaml
# Agent Field Integration Recipe
name: 'Agent Field Integration'
description: 'Integrates AI agent fields with content types'
type: 'Configuration'
# ...

config:
  # Field storage definitions
  field_storage_config:
    field_storage_config.node.field_agent_reference:
      # ...
  
  # Field instances
  field_config:
    field_config.node.ai_content.field_agent_reference:
      # ...
    
  # Entity form displays
  entity_form_display:
    # ...
    
  # Entity view displays
  entity_view_display:
    # ...

# Integration with field_eca_bridge
field_eca_bridge_integrations:
  # ...

# Integration with paragraphs
paragraph_types:
  ai_agent_component:
    # ...
```

This recipe file provides all the necessary configuration for integrating agent fields with the LLM Platform content types.

### 4. Content Types Recipe Updates

Added the agent field storage and field instance definitions to the existing content types recipe:

```yaml
# Field Storage
field.storage.node.field_agent_reference:
  createIfNotExists:
    field_name: field_agent_reference
    entity_type: node
    type: drupal_native_ai_agent_reference
    cardinality: 1
    settings: {}
    module: drupal_native_ai

# Field instance
field.field.node.ai_agent.field_agent_reference:
  createIfNotExists:
    field_name: field_agent_reference
    entity_type: node
    bundle: ai_agent
    label: 'Agent Reference'
    description: 'Reference to an AI agent configuration'
    required: true
    settings:
      allowed_capabilities:
        - text_generation
        - chat
        - function_calling
      default_options:
        temperature: 0.7
        max_tokens: 4096
```

### 5. Main Recipe Updates

Updated the main recipe.yml to include our new agent field integration recipe:

```yaml
# ┌─────────────────────────────────┐
# │     AGENTS CONFIGURATION        │
# └─────────────────────────────────┘
agents:
  - config/recipes/agents/module_agent_configuration.recipe.yml  # AI agents for module configuration
  - config/recipes/agents/agent_field_integration.recipe.yml    # AI agent field integration
```

## Integration Benefits

This integration delivers several key benefits to users of the LLM Platform:

1. **Field-Level AI Integration**: Content types can now include agent fields directly
2. **Flexible Agent Selection**: Content editors can choose from available agents with appropriate capabilities
3. **In-Context AI Execution**: AI agents can be executed directly within content view and edit forms
4. **Paragraph Integration**: Agent capabilities can be embedded within paragraph components
5. **ECA Integration**: Agent execution events can trigger event-based workflows

## Usage Examples

### Adding Agent Fields to Content Types

Content types can be configured to include agent reference fields:

```yaml
field.field.node.article.field_content_agent:
  createIfNotExists:
    field_name: field_content_agent
    entity_type: node
    bundle: article
    label: 'Content Assistant'
    description: 'AI assistant for this article'
    required: false
    settings:
      allowed_capabilities:
        - summarization
        - translation
```

### Configuring Agent Field Displays

Agent fields can be displayed in various modes:

```yaml
entity_view_display.node.article.default:
  alterExisting:
    content:
      field_content_agent:
        type: drupal_native_ai_agent_execution
        weight: 10
        region: content
        label: hidden
        settings:
          display_mode: 'card'
          show_capabilities: true
```

### Executing Agents in Field Context

The agent execution formatter allows users to execute AI operations directly within field context, with results saved and displayed.

## Next Steps

1. **Automated Testing**: Develop comprehensive integration tests
2. **User Documentation**: Create documentation for content creators
3. **Field ECA Integration**: Enhance event integration with field_eca_bridge
4. **More Field Types**: Develop additional agent-related field types
5. **UI Enhancements**: Further improve the agent field UI components

## Conclusion

With this integration, the agent field system is now fully incorporated into the LLM Platform Recipe, allowing users to leverage the power of AI agents directly within their content workflows. This represents a significant enhancement to the platform's capabilities and provides a foundation for further AI integration in the Drupal ecosystem.
