---
id: modules-development-native_ai-PROVIDER_HOOKS
---
# Provider Hooks and Alteration

This document describes the hooks and alteration points that allow other modules to integrate with and extend the Drupal Native AI provider system.

## Provider Hooks

### Hook: drupal_native_ai_provider_info_alter

Allows modules to alter provider information detected by the Provider Detection Service.

```php
/**
 * Implements hook_drupal_native_ai_provider_info_alter().
 */
function mymodule_drupal_native_ai_provider_info_alter(array &$providers) {
  // Add a custom provider.
  $providers['my_custom_provider'] = [
    'id' => 'my_custom_provider',
    'name' => t('My Custom Provider'),
    'type' => 'custom',
    'module' => 'mymodule',
    'status' => 'available',
    'description' => t('Custom AI provider implementation.'),
    'config_form' => 'mymodule.provider_config_form',
    'priority' => 60,
  ];
  
  // Modify an existing provider.
  if (isset($providers['llm.openai'])) {
    $providers['llm.openai']['description'] = t('Enhanced OpenAI integration.');
    $providers['llm.openai']['priority'] = 15; // Change priority
  }
}
```

### Hook: drupal_native_ai_operation_capabilities_alter

Allows modules to alter the capabilities required for specific operations.

```php
/**
 * Implements hook_drupal_native_ai_operation_capabilities_alter().
 */
function mymodule_drupal_native_ai_operation_capabilities_alter(array &$operation_capabilities) {
  // Add a new operation.
  $operation_capabilities['my_custom_operation'] = ['text_generation', 'custom_capability'];
  
  // Modify an existing operation's requirements.
  if (isset($operation_capabilities['generate'])) {
    $operation_capabilities['generate'][] = 'enhanced_generation';
  }
}
```

### Hook: drupal_native_ai_provider_capabilities_alter

Allows modules to alter provider capabilities.

```php
/**
 * Implements hook_drupal_native_ai_provider_capabilities_alter().
 */
function mymodule_drupal_native_ai_provider_capabilities_alter(array &$capabilities, $provider_id) {
  // Add a capability to a specific provider.
  if ($provider_id === 'drupal_native_ai') {
    $capabilities[] = 'enhanced_generation';
  }
  
  // Add a capability to a custom provider.
  if ($provider_id === 'my_custom_provider') {
    $capabilities[] = 'custom_capability';
  }
}
```

## Integration Points

### Provider Registration

To register a new provider with the Drupal Native AI system, create a provider class implementing `AiProviderInterface` and register it as a service:

```yaml
# mymodule.services.yml
services:
  mymodule.custom_provider:
    class: Drupal\mymodule\Provider\CustomProvider
    arguments: ['@config.factory', '@logger.factory', '@state']
    tags:
      - { name: ai.provider, id: 'my_custom_provider' }
```

### Provider Capabilities

Define your provider's capabilities in the `getCapabilities()` method:

```php
/**
 * {@inheritdoc}
 */
public function getCapabilities(): array {
  return [
    'text_generation',
    'custom_capability',
    'enhanced_generation',
  ];
}
```

### Operation Mapping

To add custom operations that your provider can handle, implement the provider detection alteration hook:

```php
/**
 * Implements hook_drupal_native_ai_provider_operations_alter().
 */
function mymodule_drupal_native_ai_provider_operations_alter(array &$operations, $provider_id) {
  if ($provider_id === 'my_custom_provider') {
    $operations[] = 'my_custom_operation';
  }
}
```

## Event Subscribers

### Provider Status Events

Subscribe to provider status events to react when providers are detected or configured:

```php
namespace Drupal\mymodule\EventSubscriber;

use Drupal\drupal_native_ai\Event\ProviderStatusEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event subscriber for provider status events.
 */
class ProviderStatusSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      ProviderStatusEvent::PROVIDER_DETECTED => 'onProviderDetected',
      ProviderStatusEvent::PROVIDER_CONFIGURED => 'onProviderConfigured',
    ];
  }

  /**
   * React to provider detection events.
   */
  public function onProviderDetected(ProviderStatusEvent $event) {
    $provider_id = $event->getProviderId();
    $status = $event->getStatus();
    
    // React to provider detection.
  }

  /**
   * React to provider configuration events.
   */
  public function onProviderConfigured(ProviderStatusEvent $event) {
    $provider_id = $event->getProviderId();
    $setup_type = $event->getSetupType();
    
    // React to provider configuration.
  }

}
```

## Example: Custom Provider Integration

Here's a complete example of integrating a custom provider:

1. Create the provider class:

```php
namespace Drupal\mymodule\Provider;

use Drupal\drupal_native_ai\Provider\AbstractAiProvider;
use Symfony\Component\HttpFoundation\StreamedResponse;

/**
 * Custom AI provider implementation.
 */
class CustomProvider extends AbstractAiProvider {

  /**
   * {@inheritdoc}
   */
  public function getCapabilities(): array {
    return [
      'text_generation',
      'custom_capability',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function generate(string $prompt, array $options = []): array {
    // Implementation of text generation.
    $text = "Generated text from custom provider: $prompt";
    
    return [
      'text' => $text,
      'metrics' => [
        'tokens_used' => strlen($prompt) / 4 + strlen($text) / 4,
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function generateStreamed(string $prompt, array $options = []): StreamedResponse {
    // Implementation of streamed text generation.
    // ...
  }

  /**
   * {@inheritdoc}
   */
  public function chat(array $messages, array $options = []): array {
    // Implementation of chat completion.
    // ...
  }

  /**
   * {@inheritdoc}
   */
  public function chatStreamed(array $messages, array $options = []): StreamedResponse {
    // Implementation of streamed chat completion.
    // ...
  }

}
```

2. Register the provider service:

```yaml
# mymodule.services.yml
services:
  mymodule.custom_provider:
    class: Drupal\mymodule\Provider\CustomProvider
    arguments: ['@config.factory', '@logger.factory', '@state', '@ai.streaming']
    tags:
      - { name: ai.provider, id: 'my_custom_provider' }
```

3. Implement the configuration form:

```php
namespace Drupal\mymodule\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration form for the custom provider.
 */
class CustomProviderConfigForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'mymodule_custom_provider_config_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['drupal_native_ai.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('drupal_native_ai.settings');
    $provider_config = $config->get('providers.my_custom_provider') ?? [];
    
    // Provider settings.
    $form['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Custom Provider'),
      '#default_value' => $provider_config['enabled'] ?? FALSE,
    ];
    
    // Custom settings.
    $form['custom_setting'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom Setting'),
      '#default_value' => $provider_config['custom_setting'] ?? '',
    ];
    
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('drupal_native_ai.settings');
    
    // Save provider settings.
    $config->set('providers.my_custom_provider.enabled', $form_state->getValue('enabled'));
    $config->set('providers.my_custom_provider.custom_setting', $form_state->getValue('custom_setting'));
    
    $config->save();
    
    parent::submitForm($form, $form_state);
  }

}
```

4. Add the configuration form route:

```yaml
# mymodule.routing.yml
mymodule.provider_config_form:
  path: '/admin/config/ai/native/providers/custom/configure'
  defaults:
    _form: '\Drupal\mymodule\Form\CustomProviderConfigForm'
    _title: 'Configure Custom Provider'
  requirements:
    _permission: 'administer drupal_native_ai configuration'
```

5. Implement the provider info alter hook:

```php
/**
 * Implements hook_drupal_native_ai_provider_info_alter().
 */
function mymodule_drupal_native_ai_provider_info_alter(array &$providers) {
  // Add custom provider info for detection.
  $providers['my_custom_provider'] = [
    'id' => 'my_custom_provider',
    'name' => t('My Custom Provider'),
    'type' => 'custom',
    'module' => 'mymodule',
    'status' => \Drupal::moduleHandler()->moduleExists('mymodule') ? 'available' : 'missing_dependency',
    'description' => t('Custom AI provider implementation.'),
    'config_form' => 'mymodule.provider_config_form',
    'priority' => 60,
  ];
}
```

With these components in place, your custom provider will be detected by the Provider Detection System and can be configured through the Setup Wizard or command-line tools.
