---
id: move_consolidated-CONTRIB_MODULE_INTEGRATION
---
# Contrib Module Integration Guide
*Leveraging Existing Drupal AI Modules for Private LLM Platform*

## 🎯 **Executive Summary**

We're pivoting from custom module development to leveraging the robust Drupal AI contrib ecosystem. This approach provides:
- **Faster implementation** (2 weeks vs 6+ weeks)
- **Community support** (security updates, bug fixes)
- **Proven functionality** (used by thousands of sites)
- **Complete privacy** (using Ollama for local models)

## 📦 **Required Contrib Modules**

### **Core AI Infrastructure**

#### **1. AI Module** (drupal.org/project/ai)
- **Purpose**: Core AI framework with provider abstraction
- **Version**: 1.0-beta1 or later
- **Features**:
  - Provider plugin system
  - Multiple model support
  - Standardized API
  - Caching and queuing
```bash
composer require drupal/ai:^1.0@beta
```

#### **2. Ollama Provider** (drupal.org/project/ai_provider_ollama)
- **Purpose**: Run open-source LLMs locally
- **Version**: 1.0-alpha1 or later
- **Features**:
  - Local model execution
  - No external API calls
  - Multiple model support
  - GPU acceleration
```bash
composer require drupal/ai_provider_ollama:^1.0@alpha
```

#### **3. ECA Module** (drupal.org/project/eca)
- **Purpose**: Visual workflow automation
- **Version**: 2.0 or later
- **Features**:
  - No-code workflow builder
  - 500+ actions available
  - Visual debugging
  - Import/export workflows
```bash
composer require drupal/eca:^2.0
```

#### **4. Search API AI** (drupal.org/project/search_api_ai)
- **Purpose**: Vector embeddings and RAG
- **Version**: 1.0-alpha1 or later
- **Features**:
  - Semantic search
  - Multiple vector DB support
  - OpenAI compatible embeddings
  - RAG implementation
```bash
composer require drupal/search_api_ai:^1.0@alpha
```

### **Supporting Modules**

#### **5. Key Module** (drupal.org/project/key)
- **Purpose**: Secure credential storage
- **Version**: 1.17 or later
```bash
composer require drupal/key:^1.17
```

#### **6. BPMN.io** (drupal.org/project/bpmn_io)
- **Purpose**: Visual workflow designer for ECA
- **Version**: 2.0 or later
```bash
composer require drupal/bpmn_io:^2.0
```

## 🏗️ **Integration Architecture**

```mermaid
graph TB
    A[Drupal Core] --> B[AI Module Framework]
    B --> C[Ollama Provider]
    B --> D[OpenAI Provider]
    B --> E[Custom Providers]
    
    F[ECA Module] --> G[Task Routing Workflows]
    G --> B
    
    H[Search API AI] --> I[Vector Embeddings]
    I --> J[Milvus/Pinecone/PGVector]
    
    K[Your Custom Modules] --> B
    K --> F
    K --> H
    
    L[Security Layer] --> M[Key Module]
    L --> N[Secure Drupal]
    
    style B fill:#f9f,stroke:#333,stroke-width:4px
    style C fill:#9f9,stroke:#333,stroke-width:2px
    style F fill:#9ff,stroke:#333,stroke-width:2px
```

## 📋 **Implementation Phases**

### **Phase 1: Infrastructure Setup (Days 1-3)**

1. **Install Ollama Server**
```bash
# On your AI server
curl -fsSL https://ollama.ai/install.sh | sh
systemctl enable ollama
systemctl start ollama

# Configure for network access
echo 'OLLAMA_HOST=0.0.0.0' >> /etc/environment
```

2. **Download Models**
```bash
# Task-specific models
ollama pull llama2:70b        # General content
ollama pull codellama:34b     # Code tasks
ollama pull mistral:7b        # Summarization
ollama pull nomic-embed-text  # Embeddings
```

3. **Install Drupal Modules**
```bash
cd /path/to/drupal
composer require \
  drupal/ai:^1.0@beta \
  drupal/ai_provider_ollama:^1.0@alpha \
  drupal/eca:^2.0 \
  drupal/bpmn_io:^2.0 \
  drupal/search_api_ai:^1.0@alpha \
  drupal/key:^1.17

drush en ai ai_provider_ollama eca bpmn_io search_api_ai key -y
```

### **Phase 2: Provider Configuration (Days 4-5)**

1. **Configure Ollama Connection**
   - Navigate to `/admin/config/ai/providers`
   - Add Ollama provider
   - Set endpoint: `http://your-ollama-server:11434`

2. **Create Task-Specific Providers**
```php
// Configuration example
$config['ai.provider.content'] = [
  'id' => 'content',
  'label' => 'Content Generation',
  'provider' => 'ollama',
  'model' => 'llama2:70b',
  'options' => [
    'temperature' => 0.7,
    'max_tokens' => 4096,
  ],
];
```

### **Phase 3: Workflow Creation (Days 6-8)**

1. **Design Task Routing Workflow**
   - Open BPMN.io modeler at `/admin/config/workflow/eca/models`
   - Create new model: "AI Task Router"

2. **Example Workflow Structure**
```yaml
Start Event: Entity Presave
├── Gateway: Check Task Type
│   ├── Code Analysis → Use CodeLlama Provider
│   ├── Content Generation → Use Llama2 Provider
│   └── Summarization → Use Mistral Provider
└── End Event: Save Results
```

### **Phase 4: Search Integration (Days 9-10)**

1. **Setup Vector Database**
```bash
# Using Docker for Milvus
docker-compose up -d milvus
```

2. **Configure Search API**
   - Create server with AI backend
   - Configure embeddings provider
   - Index content with vectors

### **Phase 5: Custom Module Updates (Days 11-14)**

1. **Update native_ai Module**
```php
// Before: Custom provider code
$provider = new CustomLlmProvider();

// After: Use AI module service
$provider = \Drupal::service('ai.provider_manager')
  ->createInstance('content');
```

2. **Update secure_drupal Module**
```php
// Add AI operation logging
function secure_drupal_ai_operation_alter(&$operation) {
  \Drupal::service('secure_drupal.audit')
    ->log('ai_operation', $operation);
}
```

## 🔧 **Configuration Examples**

### **ECA Workflow: Smart Content Router**
```json
{
  "id": "smart_content_router",
  "name": "Smart Content Router",
  "events": [{
    "plugin": "content_entity:presave",
    "label": "Before saving content"
  }],
  "conditions": [{
    "plugin": "eca_contains_text",
    "label": "Contains code",
    "pattern": "/(function|class|<?php|```)/",
    "field": "body"
  }],
  "actions": [{
    "plugin": "ai_generate",
    "label": "Analyze code",
    "provider": "code_analysis",
    "prompt": "Analyze this code for security issues: [entity:body:value]"
  }]
}
```

### **Search API AI Configuration**
```yaml
# config/sync/search_api.server.ai_vectors.yml
id: ai_vectors
name: 'AI Vector Search'
description: 'Semantic search with embeddings'
backend: search_api_ai
backend_config:
  embedding_provider: ollama_embeddings
  embedding_model: nomic-embed-text
  vector_database: milvus
  vector_dimensions: 384
  similarity_metric: cosine
```

## 📊 **Performance Optimization**

### **Model Loading Strategy**
```php
// Preload frequently used models
$config['ai.performance']['preload_models'] = [
  'mistral:7b',  // Fast, always loaded
  'codellama:7b', // Smaller code model for quick tasks
];

// Lazy load large models
$config['ai.performance']['lazy_load_models'] = [
  'llama2:70b',   // Load on demand
  'codellama:34b', // Load for complex tasks
];
```

### **Caching Configuration**
```php
// Cache AI responses
$config['ai.cache']['enabled'] = TRUE;
$config['ai.cache']['ttl'] = 3600; // 1 hour
$config['ai.cache']['vary_by'] = ['model', 'temperature'];
```

## 🔒 **Security Best Practices**

### **1. Network Isolation**
```bash
# Firewall rules
ufw allow from drupal_server to any port 11434
ufw deny 11434
```

### **2. Access Control**
```yaml
# permissions.yml
use ai content generation:
  title: 'Use AI for content'
  restrict access: true

use ai code analysis:
  title: 'Use AI for code'
  restrict access: true
```

### **3. Audit Logging**
```php
// Log all AI operations
function mymodule_ai_execute_alter(&$execution) {
  \Drupal::logger('ai_audit')->info('AI execution: @model', [
    '@model' => $execution['model'],
    'context' => $execution,
  ]);
}
```

## 📈 **Monitoring and Metrics**

### **Performance Metrics**
```php
// Track model performance
$metrics = \Drupal::service('ai.metrics');
$metrics->track('response_time', $model, $duration);
$metrics->track('token_usage', $model, $tokens);
```

### **Health Checks**
```bash
# Ollama health
curl http://localhost:11434/api/health

# Model availability
ollama list
```

## ✅ **Validation Checklist**

### **Infrastructure**
- [ ] Ollama server running and accessible
- [ ] All required models downloaded
- [ ] Sufficient GPU/CPU resources
- [ ] Network properly configured

### **Drupal Configuration**
- [ ] All modules installed and enabled
- [ ] Providers configured and tested
- [ ] ECA workflows created
- [ ] Search API configured
- [ ] Permissions set correctly

### **Integration**
- [ ] Custom modules updated to use AI service
- [ ] Security logging enabled
- [ ] Cache configured
- [ ] Monitoring active

### **Testing**
- [ ] Each provider returns responses
- [ ] Task routing works correctly
- [ ] Search returns relevant results
- [ ] Performance acceptable

## 🚀 **Benefits Realized**

1. **Time Saved**: 4+ weeks of development avoided
2. **Quality**: Production-tested code from contrib
3. **Support**: Community maintenance and updates
4. **Features**: More capabilities than custom build
5. **Privacy**: Complete data control with Ollama

## 📚 **Resources**

- [AI Module Documentation](https://project.pages.drupalcode.org/ai/)
- [ECA Guide](https://ecaguide.org/)
- [Ollama Documentation](https://ollama.ai/docs)
- [Search API AI Guide](https://www.drupal.org/docs/contributed-modules/search-api-ai)

## 🎯 **Success Metrics**

- **Implementation Time**: 2 weeks (target)
- **Code Coverage**: N/A (using contrib)
- **Performance**: <500ms average response
- **Uptime**: 99.9% availability
- **Cost**: $0 API fees (local models)

**The future is configuration, not code!** 🚀
