---
id: move_consolidated-DEMO_ENVIRONMENT_FIX_PLAN
---
# DEMO Environment Fix & Enhancement Plan

**Status**: Critical - Fix existing environment and preserve custom work  
**Objective**: Restore DEMO environment and build MVP using existing custom modules  
**Timeline**: 2-3 hours for complete restoration and MVP

## 🎯 Current Situation Analysis

### ✅ What We Have (Valuable Assets)
- **Custom Drupal modules**: api_normalization (95%), llm (90%), secure_drupal (85%)
- **Recipe system**: Complete recipe.yml files for automated installation
- **Field ECA Bridge**: Working integration between Field API and ECA workflows
- **LangChain integration**: PHP LangChain implementation
- **Database**: MariaDB with existing content and configuration

### ❌ Current Issues
- **Service conflicts**: Removed native_ai module still referenced in service container
- **Cache corruption**: Container compilation failing due to missing classes
- **Drush dysfunction**: Module management commands not working
- **Bootstrap errors**: Drupal unable to fully initialize

## 🔧 Phase 1: Environment Restoration (45 minutes)

### Step 1: Service Container Cleanup (15 minutes)
```bash
# 1. Clean service cache manually
ddev mysql -e "TRUNCATE cache_container; TRUNCATE cache_discovery; TRUNCATE cache_bootstrap; TRUNCATE cache_config;"

# 2. Remove problematic service definitions
find web/modules/custom -name "*.services.yml" -exec grep -l "drupal_native_ai" {} \;
# Edit files to remove references to non-existent classes

# 3. Clear file-based caches
rm -rf web/sites/default/files/php/twig/*
rm -rf web/sites/default/files/config_*
```

### Step 2: Module Dependency Resolution (15 minutes)
```bash
# 1. Audit enabled modules and dependencies
ddev mysql -e "SELECT data FROM config WHERE name = 'core.extension';" | grep -o 's:[0-9]*:"[^"]*"' | sort

# 2. Identify and resolve conflicts
# Check for missing dependencies
# Remove references to deleted modules

# 3. Rebuild module registry
ddev exec php -r "require_once 'web/autoload.php'; drupal_flush_all_caches();"
```

### Step 3: Clean Bootstrap Test (15 minutes)
```bash
# 1. Test basic Drupal bootstrap
ddev exec php -r "
require_once 'web/autoload.php';
use Drupal\Core\DrupalKernel;
use Symfony\Component\HttpFoundation\Request;
\$kernel = new DrupalKernel('prod', \$autoloader);
\$request = Request::createFromGlobals();
\$kernel->boot();
echo 'Bootstrap successful';
"

# 2. Verify Drush functionality
ddev drush status

# 3. Test module list access
ddev drush core:requirements
```

## 📋 Phase 2: Documentation Consolidation (30 minutes)

### Remaining Markdown Files Audit
```bash
# Find all remaining markdown files
find . -name "*.md" -type f \
  -not -path "*/web/core/*" \
  -not -path "*/web/modules/contrib/*" \
  -not -path "*/node_modules/*" \
  -not -path "*/./_CommonNPM/bfdocs/*" \
  | wc -l

# Categorize by location and importance
```

### Consolidation Strategy
1. **Module Documentation**: Move all module README and docs to bfdocs/modules/
2. **Implementation Guides**: Consolidate scattered implementation docs
3. **Architecture Documents**: Merge technical specs into central architecture
4. **Delete Duplicates**: Remove redundant and outdated files

### Target Structure
```
bfdocs/docs/
├── modules/
│   ├── production/
│   │   ├── api_normalization/          # Complete module docs
│   │   ├── llm/                        # LLM platform docs  
│   │   └── secure_drupal/              # Security framework docs
│   ├── development/
│   │   ├── field_eca_bridge/           # ECA integration docs
│   │   ├── langchain/                  # LangChain PHP docs
│   │   ├── drupal_migration_plus/      # Migration tools docs
│   │   └── blockchain_manager/         # Blockchain integration docs
│   └── recipes/
│       ├── llm_platform/               # Recipe documentation
│       └── installation_guides/        # Setup instructions
```

## 🚀 Phase 3: MVP Implementation Using Custom Modules (60 minutes)

### Leverage Existing Custom Modules
1. **api_normalization** (95% complete) - Use as foundation for API standardization
2. **llm module** (90% complete) - Core AI orchestration platform
3. **secure_drupal** (85% complete) - Security and compliance framework
4. **field_eca_bridge** (80% complete) - Workflow automation

### MVP Architecture Using Custom Modules
```yaml
Core Platform:
  - api_normalization: API standards and validation
  - llm: Multi-model AI provider management
  - secure_drupal: Enterprise security controls
  - field_eca_bridge: Visual workflow automation

Private LLM Integration:
  - Use llm module's provider system
  - Add Ollama provider configuration
  - Leverage existing token management
  - Utilize built-in audit logging

Content Processing:
  - field_eca_bridge: Automated content enhancement
  - api_normalization: Consistent API responses
  - secure_drupal: Compliance validation
  - llm: Model routing and management
```

### Implementation Steps
1. **Restore environment** and verify custom modules
2. **Configure AI providers** using existing llm module
3. **Set up workflows** using field_eca_bridge
4. **Test integration** with sample content
5. **Document MVP** capabilities and usage

## 📊 Phase 4: Final Integration & Documentation (45 minutes)

### Complete Documentation Migration
1. **Audit remaining files**: Identify all markdown files outside bfdocs
2. **Migrate valuable content**: Move important docs to appropriate bfdocs locations
3. **Remove duplicates**: Delete redundant and outdated files
4. **Update cross-references**: Fix all links to point to central documentation
5. **Create final index**: Master navigation for the complete platform

### MVP Documentation Deliverables
1. **Working Demo Guide**: Step-by-step usage of restored environment
2. **Module Integration Map**: How custom modules work together
3. **API Documentation**: Complete OpenAPI specs using api_normalization
4. **Security Guide**: Using secure_drupal for compliance
5. **Workflow Examples**: ECA configurations for common AI tasks

## 🛠️ Execution Plan

### Immediate Actions (Next 15 minutes)
1. **Diagnose service issues**: Identify specific service conflicts
2. **Clean cache tables**: Manual database cache clearing
3. **Test bootstrap**: Verify basic Drupal functionality
4. **Document findings**: Record specific issues and solutions

### Recovery Strategy
- **Conservative approach**: Fix issues incrementally
- **Preserve custom work**: Never delete custom modules or recipes
- **Test frequently**: Verify each fix before proceeding
- **Document process**: Create restoration guide for future reference

### Success Criteria
- ✅ Drupal bootstrap successful
- ✅ Drush commands working
- ✅ Custom modules functional
- ✅ Recipe system operational
- ✅ Documentation consolidated
- ✅ MVP demo working

---

**Next Action**: Start with service container cleanup and diagnose specific bootstrap issues
