---
id: move_consolidated-MILVUS_VECTOR_DB_INTEGRATION
---
# 🚀 Milvus Vector Database Integration Guide

## Overview

Milvus is the world's most advanced open-source vector database, designed specifically for high-performance vector similarity search and AI applications. This guide integrates Milvus with your private AI ecosystem for optimal vector storage and retrieval.

## 🎯 Why Milvus for Vector Storage

### **Milvus vs Other Vector Databases**

| Feature | **Milvus** | pgvector | Pinecone | Weaviate |
|---------|------------|----------|----------|----------|
| **Performance** | ✅ **10K+ QPS** | ⚠️ 100-1K QPS | ✅ 10K+ QPS | ✅ 5K+ QPS |
| **Scale** | ✅ **10B+ vectors** | ⚠️ 1M vectors | ✅ 5B+ vectors | ✅ 1B+ vectors |
| **Privacy** | ✅ **Self-hosted** | ✅ Self-hosted | ❌ Cloud only | ✅ Self-hosted |
| **Cost** | ✅ **$0 per vector** | ✅ $0 per vector | ❌ $0.096/1K vectors | ⚠️ Infrastructure |
| **Index Types** | ✅ **8+ algorithms** | ⚠️ 2 algorithms | ✅ Multiple | ✅ Multiple |
| **Kubernetes** | ✅ **Native Helm** | ⚠️ Manual | ❌ Not applicable | ✅ Helm charts |

### **Milvus Advantages for Private AI**

- ✅ **100% Private**: Complete data sovereignty with self-hosting
- ✅ **Kubernetes Native**: Deploy with official Helm charts
- ✅ **High Performance**: Sub-millisecond vector queries at scale
- ✅ **Multiple Indexes**: FLAT, IVF, HNSW, ANNOY for different use cases
- ✅ **Horizontal Scaling**: Auto-scale based on workload
- ✅ **Enterprise Ready**: Production-grade with monitoring and backup

## 🛠️ Milvus Deployment with Helm

### Step 1: Add Milvus Helm Repository

```bash
# Add the Milvus Helm repository
helm repo add milvus https://zilliztech.github.io/milvus-helm/
helm repo update

# Verify the chart is available
helm search repo milvus
```

### Step 2: Create Custom Values File

```yaml
# milvus-values.yaml - Production configuration
cluster:
  enabled: true

# Milvus components configuration
milvus:
  image:
    repository: milvusdb/milvus
    tag: v2.3.0
    pullPolicy: IfNotPresent

# Coordinators (metadata management)
rootCoordinator:
  replicas: 1
  resources:
    requests:
      cpu: "0.1"
      memory: "512Mi"
    limits:
      cpu: "1"
      memory: "2Gi"

dataCoordinator:
  replicas: 1
  resources:
    requests:
      cpu: "0.1"
      memory: "256Mi"
    limits:
      cpu: "0.5"
      memory: "1Gi"

queryCoordinator:
  replicas: 1
  resources:
    requests:
      cpu: "0.1"
      memory: "256Mi"
    limits:
      cpu: "0.5"
      memory: "1Gi"

indexCoordinator:
  replicas: 1
  resources:
    requests:
      cpu: "0.1"
      memory: "256Mi"
    limits:
      cpu: "0.5"
      memory: "1Gi"

# Worker nodes (data processing)
proxy:
  replicas: 2
  resources:
    requests:
      cpu: "0.2"
      memory: "512Mi"
    limits:
      cpu: "1"
      memory: "2Gi"
  service:
    type: ClusterIP
    port: 19530

queryNode:
  replicas: 2
  resources:
    requests:
      cpu: "0.5"
      memory: "1Gi"
    limits:
      cpu: "2"
      memory: "4Gi"

dataNode:
  replicas: 2
  resources:
    requests:
      cpu: "0.2"
      memory: "512Mi"
    limits:
      cpu: "1"
      memory: "2Gi"

indexNode:
  replicas: 1
  resources:
    requests:
      cpu: "0.5"
      memory: "1Gi"
    limits:
      cpu: "4"
      memory: "8Gi"

# External dependencies
etcd:
  enabled: true
  replicaCount: 3
  persistence:
    enabled: true
    size: 10Gi
    storageClass: "fast-ssd"
  resources:
    requests:
      cpu: "0.1"
      memory: "256Mi"
    limits:
      cpu: "0.5"
      memory: "1Gi"

minio:
  enabled: true
  mode: distributed
  replicas: 4
  persistence:
    enabled: true
    size: 50Gi
    storageClass: "fast-ssd"
  resources:
    requests:
      cpu: "0.2"
      memory: "512Mi"
    limits:
      cpu: "1"
      memory: "2Gi"

pulsar:
  enabled: true
  components:
    broker: true
    bookkeeper: true
    zookeeper: true
  broker:
    replicaCount: 2
    resources:
      requests:
        cpu: "0.2"
        memory: "512Mi"
      limits:
        cpu: "1"
        memory: "2Gi"
  bookkeeper:
    replicaCount: 3
    resources:
      requests:
        cpu: "0.2"
        memory: "512Mi"
      limits:
        cpu: "1"
        memory: "2Gi"
  zookeeper:
    replicaCount: 3
    resources:
      requests:
        cpu: "0.1"
        memory: "256Mi"
      limits:
        cpu: "0.5"
        memory: "1Gi"

# Monitoring and observability
metrics:
  enabled: true
  serviceMonitor:
    enabled: true
    interval: 30s

log:
  level: info
  format: text

# Security configurations
security:
  tls:
    enabled: false
  authz:
    enabled: false

# Custom configurations
config:
  common:
    retentionDuration: 432000  # 5 days in seconds
    indexSliceSize: 4          # GB
  queryNode:
    enableDisk: true
    diskPath: /tmp/milvus/disk
```

### Step 3: Deploy Milvus

```bash
# Create namespace for Milvus
kubectl create namespace milvus

# Deploy Milvus with custom values
helm install milvus milvus/milvus \
  --namespace milvus \
  --values milvus-values.yaml \
  --wait \
  --timeout 600s

# Verify deployment
kubectl get pods -n milvus
kubectl get services -n milvus
```

### Step 4: Verify Installation

```bash
# Forward port for testing
kubectl port-forward -n milvus service/milvus 19530:19530 &

# Test connection with Python client
python3 -c "
from pymilvus import connections, Collection
connections.connect('default', host='localhost', port='19530')
print('✅ Milvus connection successful!')
"
```

## 🔧 Drupal Integration Setup

### Step 1: Create Milvus Integration Module

```bash
# Navigate to Drupal custom modules
cd ./DEMO/web/modules/custom

# Create the Milvus integration module
mkdir -p milvus_integration/src/Service
mkdir -p milvus_integration/src/Plugin/search_api/backend
```

### Step 2: Module Configuration

```yaml
# milvus_integration.info.yml
name: 'Milvus Vector Database Integration'
type: module
description: 'High-performance vector search using Milvus database for AI-powered content discovery.'
core_version_requirement: ^10 || ^11
package: 'AI Platform - Vector Search'
version: '1.0.0'

dependencies:
  - drupal:system
  - search_api:search_api
  - ai:ai
  - private_llm_provider:private_llm_provider

# Vector search features
features:
  - vector_similarity_search
  - semantic_content_discovery
  - real_time_embeddings
  - high_performance_indexing
  - horizontal_scaling
```

### Step 3: Milvus Client Service

```php
<?php
// src/Service/MilvusClient.php

namespace Drupal\milvus_integration\Service;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use GuzzleHttp\ClientInterface;

/**
 * Milvus client service for vector operations.
 */
class MilvusClient {

  protected string $host;
  protected int $port;
  protected ClientInterface $httpClient;
  protected LoggerChannelFactoryInterface $loggerFactory;

  public function __construct(array $connection, LoggerChannelFactoryInterface $logger_factory, ClientInterface $http_client) {
    $this->host = $connection['host'] ?? 'milvus.milvus.svc.cluster.local';
    $this->port = $connection['port'] ?? 19530;
    $this->loggerFactory = $logger_factory;
    $this->httpClient = $http_client;
  }

  /**
   * Creates a collection for vector storage.
   */
  public function createCollection(string $name, int $dimension, string $metric = 'COSINE'): bool {
    $payload = [
      'collection_name' => $name,
      'schema' => [
        'autoID' => true,
        'description' => "Collection for {$name} vectors",
        'fields' => [
          [
            'fieldName' => 'id',
            'dataType' => 'Int64',
            'isPrimary' => true,
            'autoID' => true,
          ],
          [
            'fieldName' => 'vector',
            'dataType' => 'FloatVector',
            'typeParams' => ['dim' => $dimension],
          ],
          [
            'fieldName' => 'entity_id',
            'dataType' => 'Int64',
          ],
          [
            'fieldName' => 'entity_type',
            'dataType' => 'VarChar',
            'typeParams' => ['max_length' => 64],
          ],
        ],
      ],
    ];

    return $this->makeRequest('POST', '/collections', $payload);
  }

  /**
   * Inserts vectors into a collection.
   */
  public function insertVectors(string $collection, array $vectors, array $metadata): array {
    $payload = [
      'collection_name' => $collection,
      'fields_data' => [
        [
          'field_name' => 'vector',
          'type' => 101, // FloatVector
          'field' => $vectors,
        ],
        [
          'field_name' => 'entity_id',
          'type' => 5, // Int64
          'field' => $metadata['entity_ids'],
        ],
        [
          'field_name' => 'entity_type',
          'type' => 21, // VarChar
          'field' => $metadata['entity_types'],
        ],
      ],
    ];

    return $this->makeRequest('POST', '/entities', $payload);
  }

  /**
   * Searches for similar vectors.
   */
  public function searchVectors(string $collection, array $query_vector, int $limit = 10, float $threshold = 0.7): array {
    $payload = [
      'collection_name' => $collection,
      'vectors' => [$query_vector],
      'search_params' => [
        'anns_field' => 'vector',
        'topk' => $limit,
        'params' => ['nprobe' => 16],
        'metric_type' => 'COSINE',
      ],
      'output_fields' => ['entity_id', 'entity_type'],
    ];

    $result = $this->makeRequest('POST', '/search', $payload);
    
    // Filter by threshold
    if (isset($result['results'])) {
      foreach ($result['results'] as &$batch) {
        $batch = array_filter($batch, function($item) use ($threshold) {
          return $item['score'] >= $threshold;
        });
      }
    }

    return $result;
  }

  /**
   * Creates an index on a collection for better performance.
   */
  public function createIndex(string $collection, string $field = 'vector', string $index_type = 'IVF_FLAT'): bool {
    $payload = [
      'collection_name' => $collection,
      'field_name' => $field,
      'index_name' => "{$field}_index",
      'extra_params' => [
        'index_type' => $index_type,
        'metric_type' => 'COSINE',
        'params' => ['nlist' => 1024],
      ],
    ];

    return $this->makeRequest('POST', '/indexes', $payload);
  }

  /**
   * Makes HTTP request to Milvus API.
   */
  protected function makeRequest(string $method, string $endpoint, array $data = []): array {
    try {
      $response = $this->httpClient->request($method, "http://{$this->host}:9091{$endpoint}", [
        'json' => $data,
        'timeout' => 30,
      ]);

      $result = json_decode($response->getBody()->getContents(), true);
      
      if ($result['code'] !== 0) {
        throw new \Exception("Milvus error: {$result['message']}");
      }

      return $result['data'] ?? $result;

    } catch (\Exception $e) {
      $this->loggerFactory->get('milvus')->error('Milvus request failed: @error', [
        '@error' => $e->getMessage(),
      ]);
      throw $e;
    }
  }
}
```

### Step 4: Search API Backend Integration

```php
<?php
// src/Plugin/search_api/backend/MilvusBackend.php

namespace Drupal\milvus_integration\Plugin\search_api\backend;

use Drupal\search_api\Backend\BackendPluginBase;
use Drupal\search_api\IndexInterface;
use Drupal\search_api\Item\ItemInterface;
use Drupal\search_api\Query\QueryInterface;
use Drupal\search_api\SearchApiException;

/**
 * Milvus backend for Search API.
 *
 * @SearchApiBackend(
 *   id = "milvus",
 *   label = @Translation("Milvus Vector Database"),
 *   description = @Translation("High-performance vector search using Milvus database.")
 * )
 */
class MilvusBackend extends BackendPluginBase {

  /**
   * {@inheritdoc}
   */
  public function addIndex(IndexInterface $index) {
    $collection_name = $this->getCollectionName($index);
    
    // Create collection in Milvus
    $this->getMilvusClient()->createCollection($collection_name, 768);
    $this->getMilvusClient()->createIndex($collection_name);
    
    return parent::addIndex($index);
  }

  /**
   * {@inheritdoc}
   */
  public function indexItems(IndexInterface $index, array $items) {
    $vectors = [];
    $metadata = ['entity_ids' => [], 'entity_types' => []];
    
    foreach ($items as $item) {
      // Extract text content and generate embeddings
      $text = $this->extractTextFromItem($item);
      $vector = $this->generateEmbedding($text);
      
      $vectors[] = $vector;
      $metadata['entity_ids'][] = (int) $item->getId();
      $metadata['entity_types'][] = $item->getDatasource()->getEntityTypeId();
    }
    
    // Insert into Milvus
    $collection_name = $this->getCollectionName($index);
    $result = $this->getMilvusClient()->insertVectors($collection_name, $vectors, $metadata);
    
    return array_keys($items);
  }

  /**
   * {@inheritdoc}
   */
  public function search(QueryInterface $query) {
    $search_text = $query->getOriginalKeys();
    if (empty($search_text)) {
      return $query->getResults();
    }

    // Generate embedding for search query
    $query_vector = $this->generateEmbedding($search_text);
    
    // Search in Milvus
    $collection_name = $this->getCollectionName($query->getIndex());
    $results = $this->getMilvusClient()->searchVectors(
      $collection_name, 
      $query_vector, 
      $query->getOption('limit', 10)
    );

    // Convert Milvus results to Search API format
    return $this->convertMilvusResults($query, $results);
  }

  /**
   * Generates embeddings using Ollama.
   */
  protected function generateEmbedding(string $text): array {
    $embedding_service = \Drupal::service('private_llm_provider.data_sanitization');
    $sanitized_text = $embedding_service->sanitizeText($text);
    
    $ai_provider = \Drupal::service('ai.provider')->createInstance('ollama');
    $embedding_input = new \Drupal\ai\OperationType\Embeddings\EmbeddingsInput($sanitized_text);
    $result = $ai_provider->embeddings($embedding_input, ['model' => 'nomic-embed-text']);
    
    return $result->getEmbeddings()[0];
  }

  /**
   * Gets Milvus client service.
   */
  protected function getMilvusClient() {
    return \Drupal::service('milvus.client');
  }

  /**
   * Gets collection name for an index.
   */
  protected function getCollectionName(IndexInterface $index): string {
    return 'drupal_' . $index->id();
  }
}
```

## 🚀 Performance Optimization

### Collection Configuration for Different Use Cases

```yaml
# High-performance configurations for different scenarios

content_embeddings:
  index_type: "IVF_FLAT"
  params:
    nlist: 1024      # Number of clusters
    nprobe: 16       # Number of clusters to search
  metric: "COSINE"
  dimensions: 768
  use_case: "General content similarity"

code_embeddings:
  index_type: "HNSW"
  params:
    M: 16            # Max connections per node
    efConstruction: 200  # Size of dynamic candidate list
    ef: 64           # Search scope
  metric: "IP"       # Inner Product for code similarity
  dimensions: 768
  use_case: "Code analysis and similarity"

semantic_cache:
  index_type: "IVF_SQ8"
  params:
    nlist: 2048
    nprobe: 32
  metric: "COSINE"
  dimensions: 1536
  use_case: "LLM response caching"

user_behavior:
  index_type: "FLAT"
  metric: "L2"
  dimensions: 384
  use_case: "Small dataset, perfect recall needed"
```

### Monitoring and Alerting

```yaml
# Prometheus metrics for Milvus
milvus_metrics:
  - milvus_proxy_req_count
  - milvus_proxy_req_latency
  - milvus_querynode_load_memory_usage
  - milvus_datanode_flowgraph_consume_time
  - milvus_indexnode_index_task_count

grafana_alerts:
  - name: "MilvusHighLatency"
    condition: "milvus_proxy_req_latency > 100ms"
    action: "Scale query nodes"
  
  - name: "MilvusMemoryUsage"
    condition: "milvus_querynode_memory > 80%"
    action: "Scale horizontally"
    
  - name: "MilvusConnectionFailure"
    condition: "milvus_proxy_connection_errors > 5"
    action: "Alert ops team"
```

## 📊 Performance Benchmarks

### Expected Performance Metrics

```yaml
Vector Operations:
  Insert Rate: 50,000 vectors/second
  Query Latency: <10ms (95th percentile)
  Query Throughput: 10,000+ QPS
  Index Build Time: 100M vectors in <30 minutes

Resource Usage:
  Memory: ~2GB per 1M vectors (HNSW)
  Memory: ~1GB per 1M vectors (IVF_FLAT)
  Storage: ~4KB per vector (768 dimensions)
  CPU: 2-4 cores per query node

Scaling Characteristics:
  Collections: Unlimited
  Vectors per Collection: 10B+
  Concurrent Queries: 1000+
  Horizontal Scaling: Linear performance
```

## 🔧 Maintenance and Backup

### Backup Strategy

```bash
# Backup Milvus data (via MinIO)
kubectl exec -n milvus milvus-minio-0 -- \
  mc mirror milvus-bucket /backup/milvus-$(date +%Y%m%d)

# Backup metadata (via etcd)
kubectl exec -n milvus milvus-etcd-0 -- \
  etcdctl snapshot save /backup/etcd-$(date +%Y%m%d).db
```

### Health Checks

```bash
# Collection health check
python3 -c "
from pymilvus import connections, utility
connections.connect('default', host='milvus.milvus.svc.cluster.local', port='19530')
collections = utility.list_collections()
for col in collections:
    stats = utility.get_collection_stats(col)
    print(f'{col}: {stats}')
"
```

## 🎯 Integration with Your NPM Packages

Your NPM packages can now leverage Milvus for enhanced vector operations:

### bfapi Integration
```typescript
// Enhanced API endpoints with Milvus vector search
app.post('/api/v1/semantic-search', async (req, res) => {
  const { query, limit = 10 } = req.body;
  
  // Generate embedding via Ollama
  const embedding = await generateEmbedding(query);
  
  // Search in Milvus
  const results = await milvusClient.search('content_embeddings', embedding, limit);
  
  res.json({ results, performance: 'sub-10ms' });
});
```

### bfcli Commands
```bash
# CLI commands for Milvus management
bfcli milvus:status                    # Check cluster health
bfcli milvus:collections               # List collections
bfcli milvus:reindex content_embeddings # Rebuild indexes
bfcli milvus:backup                    # Create backup
```

## ✅ Deployment Checklist

- [ ] **Kubernetes cluster** with sufficient resources (CPU: 8+, Memory: 32GB+)
- [ ] **Storage class** configured for persistent volumes
- [ ] **Helm 3.x** installed and configured
- [ ] **Milvus Helm chart** deployed with custom values
- [ ] **Drupal integration module** enabled and configured
- [ ] **Ollama embedding models** downloaded and running
- [ ] **Collections created** for your content types
- [ ] **Indexes built** for optimal performance
- [ ] **Monitoring** configured with Prometheus/Grafana
- [ ] **Backup strategy** implemented and tested

**Your high-performance vector database is now ready for enterprise AI workloads!** 🎉

This Milvus integration provides the foundation for advanced semantic search, recommendation systems, and RAG (Retrieval-Augmented Generation) capabilities while maintaining complete data sovereignty and privacy.
