---
id: move_consolidated-PRIVATE_AI_SETUP_GUIDE
---
# 🔒 Private AI Setup Guide

## Overview
This guide helps you set up completely private AI processing using Ollama, ensuring **zero external data transmission** and maximum security.

## 🚀 Quick Setup (5 minutes)

### Step 1: Install Ollama
```bash
# On macOS/Linux
curl -fsSL https://ollama.ai/install.sh | sh

# On Windows (PowerShell)
# Download from: https://ollama.ai/download
```

### Step 2: Start Ollama Service
```bash
# Start the Ollama service
ollama serve

# Verify it's running (should return version info)
curl http://localhost:11434/api/version
```

### Step 3: Download AI Models
```bash
# Fast, general-purpose model (4GB)
ollama pull llama3.1:8b

# High-quality model (requires more GPU memory - 40GB)
ollama pull llama3.1:70b

# Code generation specialist (4GB)
ollama pull codellama:7b

# Multilingual, efficient model (4GB)  
ollama pull mistral:7b

# Text embeddings for similarity/search (1GB)
ollama pull nomic-embed-text
```

### Step 4: Enable Private LLM Provider in Drupal
```bash
cd /path/to/your/drupal
ddev drush en private_llm_provider key encrypt -y
```

### Step 5: Configure in Drupal Admin
1. Go to: `/admin/config/ai/providers`
2. Select "Ollama (Private/Local)"
3. Set Host: `http://localhost:11434`
4. Test connection
5. Save configuration

## 🛡️ Security Benefits

### Complete Data Sovereignty
- ✅ **Zero External Calls**: All processing happens locally
- ✅ **No API Keys Required**: No third-party authentication
- ✅ **No Data Logging**: Ollama doesn't log conversations
- ✅ **Offline Capable**: Works without internet connection
- ✅ **GDPR/HIPAA Ready**: Full compliance possible

### Enhanced Privacy Features
- ✅ **Automatic PII Detection**: Emails, SSNs, phone numbers masked
- ✅ **Custom Sanitization**: Define your own sensitive patterns
- ✅ **Risk Assessment**: Real-time safety validation
- ✅ **Audit Logging**: Track all privacy actions
- ✅ **Encrypted Storage**: Sensitive configs encrypted at rest

## 💰 Cost Comparison

| Provider | Cost per 1M tokens | Data Privacy | Setup Complexity |
|----------|-------------------|--------------|------------------|
| **Ollama (Private)** | **$0** | **Maximum** | **Medium** |
| OpenAI GPT-4 | $30 | Minimal | Low |
| Anthropic Claude | $15 | Minimal | Low |
| Azure OpenAI | $30 | Medium | High |

## 🖥️ Hardware Requirements

### Minimum (CPU Only)
- **RAM**: 8GB
- **Storage**: 10GB free
- **Models**: llama3.1:8b, mistral:7b
- **Performance**: ~1-2 tokens/second

### Recommended (GPU Accelerated)
- **GPU**: NVIDIA RTX 4090, A100, or similar
- **VRAM**: 24GB+ for larger models
- **RAM**: 32GB
- **Storage**: 100GB+ SSD
- **Performance**: ~50-100 tokens/second

### Production (Multi-GPU)
- **GPUs**: Multiple NVIDIA A100/H100
- **VRAM**: 80GB+ total
- **RAM**: 128GB+
- **Storage**: 1TB+ NVMe SSD
- **Performance**: ~200+ tokens/second

## 🔧 Advanced Configuration

### GPU Acceleration
```bash
# Verify GPU support
ollama run llama3.1:8b "Test GPU acceleration"

# Monitor GPU usage
nvidia-smi -l 1
```

### Custom Model Fine-tuning
```bash
# Create custom model for your domain
ollama create my-custom-model -f ./Modelfile

# Modelfile example:
FROM llama3.1:8b
PARAMETER temperature 0.1
SYSTEM "You are a helpful assistant specialized in [your domain]."
```

### Production Deployment
```yaml
# docker-compose.yml for production
version: '3.8'
services:
  ollama:
    image: ollama/ollama:latest
    ports:
      - "11434:11434"
    volumes:
      - ollama:/root/.ollama
    deploy:
      resources:
        reservations:
          devices:
            - driver: nvidia
              count: all
              capabilities: [gpu]
volumes:
  ollama:
```

## 🔍 Monitoring & Maintenance

### Health Checks
```bash
# Check model status
ollama list

# Test model performance
time ollama run llama3.1:8b "Hello world"

# Monitor resource usage
docker stats ollama
```

### Backup Strategy
```bash
# Backup model data
cp -r ~/.ollama/models /backup/ollama-models

# Backup configurations
cp -r ~/.ollama/config /backup/ollama-config
```

## 🚨 Troubleshooting

### Common Issues

1. **Ollama won't start**
   ```bash
   # Check if port is available
   lsof -i :11434
   
   # Start with debug logging
   OLLAMA_DEBUG=1 ollama serve
   ```

2. **Model loading fails**
   ```bash
   # Check disk space
   df -h
   
   # Re-download model
   ollama pull llama3.1:8b
   ```

3. **Slow performance**
   ```bash
   # Check if GPU is being used
   nvidia-smi
   
   # Reduce model size
   ollama pull llama3.1:8b  # instead of 70b
   ```

## 📊 Privacy Audit Checklist

- [ ] Ollama running locally (not cloud)
- [ ] No external API calls in logs
- [ ] PII sanitization enabled
- [ ] Custom patterns configured
- [ ] Audit logging active
- [ ] Model data encrypted
- [ ] Network isolation configured
- [ ] Backup strategy implemented

## 🎯 Next Steps

1. **Test Private Processing**: Send a sample request
2. **Configure Sanitization**: Add your custom PII patterns
3. **Monitor Performance**: Check response times and accuracy
4. **Scale Hardware**: Upgrade GPU for better performance
5. **Implement Backup**: Ensure model and config safety

**Your AI is now completely private and secure! 🎉**
