---
id: platform-archived-CUSTOM_TO_CONTRIB_MIGRATION_PLAN
---
# Custom to Contrib Module Migration Plan

## Overview
This document outlines the migration plan for replacing custom Drupal modules with contrib alternatives in the DEMO project, as identified in REALITY_CHECK.md.

## Priority 1: Drupal Migration Plus → migrate_plus + migrate_tools (100% replacement)

### Current State
- **Custom Module**: `drupal_migration_plus` (located at `/web/modules/custom/drupal_migration_plus/`)
- **Already Installed**: 
  - drupal/migrate_plus 6.0.5
  - drupal/migrate_tools 6.0.5

### Analysis
The custom `drupal_migration_plus` module provides:
1. AI-powered field mapping and migration intelligence
2. Multi-CMS source support (Drupal 7, WordPress, Sitecore, AEM, HTML)
3. Self-healing capabilities
4. Recipe system for reusable migration patterns
5. Real-time progress monitoring
6. Migration clustering
7. Parallel migration execution
8. Visual field mapping interface

### Migration Steps

#### Phase 1: Core Migration Functionality (Day 1)
1. **Backup current configuration**
   ```bash
   drush config:export
   cp -r web/sites/default/config/sync web/sites/default/config/backup-$(date +%Y%m%d)
   ```

2. **Map basic migration functionality**
   - Replace custom migration sources with migrate_plus source plugins
   - Convert custom process plugins to migrate_plus process plugins
   - Update migration group configurations

3. **Update migration YAML files**
   - Convert custom source adapters to migrate_plus data parsers (JSON, XML, CSV)
   - Replace custom field mapping with migrate_plus field plugins

#### Phase 2: Advanced Features Migration (Day 2)
1. **AI/Intelligence Features** → Consider:
   - drupal/ai module for AI provider integration
   - Custom lightweight service for field mapping suggestions
   - Remove over-engineered "self-healing" features

2. **Multi-CMS Support** → Use migrate_plus with:
   - migrate_source_csv for CSV imports
   - migrate_plus JSON/XML parsers for API imports
   - wordpress_migrate module for WordPress migrations

3. **Monitoring & Analytics** → Replace with:
   - migrate_tools drush commands for progress
   - Simple Views for migration status dashboard
   - Core logging for migration tracking

4. **Recipe System** → Simplify to:
   - Migration templates as example YAMLs
   - Documentation for common patterns

### Code Changes Required
```php
// Before (custom)
$migration_clusterer = \Drupal::service('drupal_migration_plus.migration_clusterer');
$clusters = $migration_clusterer->clusterMigrations($migrations);

// After (contrib)
$migration_plugin_manager = \Drupal::service('plugin.manager.migration');
$migrations = $migration_plugin_manager->createInstances([]);
```

### Configuration Updates
Update all migration configurations from:
```yaml
source:
  plugin: drupal_migration_plus_drupal7
  source_adapter: drupal7
```

To:
```yaml
source:
  plugin: d7_node
  # Or for external sources:
  plugin: url
  data_fetcher_plugin: http
  data_parser_plugin: json
```

---

## Priority 2: Recipe Onboarding → Core Tour Module

### Current State
- **Custom Module**: `recipe_onboarding` (located at `/web/modules/custom/recipe_onboarding/`)
- **Replacement**: Core Tour module (already in Drupal core)

### Analysis
The custom `recipe_onboarding` module provides:
1. Wizard-based onboarding interface
2. Recipe extension framework
3. Environment detection
4. Heavy dependencies (40+ modules)

### Migration Steps (1 Day)

1. **Enable Tour module**
   ```bash
   drush en tour -y
   ```

2. **Convert wizards to tours**
   - Create tour.tour.*.yml files for each onboarding flow
   - Map wizard steps to tour tips
   - Use tour's built-in progress tracking

3. **Simplify dependencies**
   - Remove unnecessary module dependencies
   - Keep only essential functionality

4. **Create tour configurations**
   ```yaml
   # Example: tour.tour.llm_platform_setup.yml
   id: llm_platform_setup
   label: 'LLM Platform Setup'
   module: tour
   routes:
     - route_name: system.admin
   tips:
     introduction:
       id: introduction
       plugin: text
       label: 'Welcome to LLM Platform'
       body: 'This tour will guide you through setting up the LLM Platform.'
       weight: 1
   ```

5. **Update module dependencies**
   - Remove recipe_onboarding from all module .info.yml files
   - Update any code that calls recipe_onboarding services

---

## Implementation Timeline

### Day 1: Morning (2-3 hours)
- Backup all configurations
- Document current custom module usage
- Create test migrations with contrib modules

### Day 1: Afternoon (3-4 hours)  
- Migrate basic drupal_migration_plus functionality
- Test core migration features
- Update migration configurations

### Day 2: Morning (2-3 hours)
- Migrate recipe_onboarding to Tour module
- Create tour configurations
- Test onboarding flows

### Day 2: Afternoon (2-3 hours)
- Clean up and remove custom modules
- Update documentation
- Final testing and validation

---

## Testing Checklist

### Migration Module Testing
- [ ] Basic node migrations work
- [ ] Field mappings are preserved
- [ ] Migration groups function correctly
- [ ] Drush commands work (`drush migrate:status`, `drush migrate:import`)
- [ ] No PHP errors in logs

### Tour Module Testing  
- [ ] Tours display on appropriate pages
- [ ] Tour tips show correct content
- [ ] Navigation between tips works
- [ ] Tours can be completed and dismissed
- [ ] No JavaScript errors

---

## Rollback Plan

If issues arise:
1. Restore configuration backup
2. Re-enable custom modules
3. Clear caches: `drush cr`
4. Document specific issues for resolution

---

## Post-Migration Cleanup

1. **Move custom modules to archive**
   ```bash
   mkdir -p web/modules/custom/__DELETE_LATER
   mv web/modules/custom/drupal_migration_plus web/modules/custom/__DELETE_LATER/drupal_migration_plus_$(date +%Y-%m-%d)
   mv web/modules/custom/recipe_onboarding web/modules/custom/__DELETE_LATER/recipe_onboarding_$(date +%Y-%m-%d)
   ```

2. **Update composer.json**
   - Ensure migrate_plus and migrate_tools are in require section
   - Remove any custom module autoload paths

3. **Clear all caches**
   ```bash
   drush cr
   drush cron
   ```

---

## Success Metrics

1. All existing migrations continue to function
2. No custom module dependencies remain
3. Reduced code complexity by ~80%
4. Improved maintainability with contrib modules
5. Easier upgrades to future Drupal versions

---

## Notes

- The custom modules are significantly over-engineered for most use cases
- Contrib modules provide 90% of needed functionality
- Remaining 10% can be handled with simple custom plugins/services if needed
- This migration aligns with Drupal best practices of using contrib when possible
