---
title: Installation Guide - Unified Compliance Framework
description: >-
  Step-by-step installation instructions for the Unified Compliance and Security
  Framework
id: unified-compliance-installation
---

# Installation Guide

This guide provides step-by-step instructions for installing and configuring the Unified Compliance and Security Framework.

## Prerequisites

Before installation, ensure you have:

- Node.js v16.0+ and npm v8.0+
- Drupal 11.0+ with Composer
- Git access to the required repositories
- PostgreSQL 14+ (for audit storage)
- Redis (for caching)
- GitLab/GitHub access tokens with API permissions

## Installation Steps

### 1. Install npm packages

First, install the core npm packages:

```bash
# Clone repositories
git clone https://gitlab.bluefly.io/llm/secure_project.git
git clone https://gitlab.bluefly.io/llm/codeowners-intelligence.git

# Install secure_project
cd secure_project
npm install
npm run build

# Install codeowners-intelligence
cd ../codeowners-intelligence
npm install
npm run build
```

### 2. Install Drupal Module

Install the Unified Compliance Security module in your Drupal installation:

```bash
# Navigate to your Drupal modules directory
cd /path/to/drupal/modules

# Create symbolic link to the unified_compliance_security module
ln -s /path/to/llm_platform_recipe/modules/unified_compliance_security unified_compliance_security

# Enable the module in Drupal
drush en unified_compliance_security
```

### 3. Configure the Bridge

Set up the Node.js bridge that connects Drupal with the TypeScript implementation:

```bash
# Make the bridge script executable
chmod +x /path/to/llm_platform_recipe/bridge/unified-compliance-bridge.js

# Configure environment variables (or use .env file)
export GITLAB_URL="https://gitlab.yourcompany.com"
export GITLAB_TOKEN="your-gitlab-token"
export GITLAB_PROJECT_ID="your-project-id"
export AUDIT_SECRET_KEY="your-secret-key"
```

For production deployments, create a proper systemd service:

```bash
# Create a systemd service file
sudo nano /etc/systemd/system/unified-compliance-bridge.service

# Add the following content
[Unit]
Description=Unified Compliance Bridge
After=network.target

[Service]
User=www-data
Group=www-data
WorkingDirectory=/path/to/llm_platform_recipe/bridge
ExecStart=/usr/bin/node unified-compliance-bridge.js
Restart=on-failure
Environment=NODE_ENV=production
Environment=GITLAB_URL=https://gitlab.yourcompany.com
Environment=GITLAB_TOKEN=your-gitlab-token
Environment=GITLAB_PROJECT_ID=your-project-id
Environment=AUDIT_SECRET_KEY=your-secret-key

[Install]
WantedBy=multi-user.target

# Enable and start the service
sudo systemctl enable unified-compliance-bridge
sudo systemctl start unified-compliance-bridge
```

### 4. Configure Drupal Settings

In Drupal, navigate to `/admin/config/security/compliance` and configure:

- Path to Node.js executable
- Path to bridge script
- GitLab/GitHub integration settings
- Security and compliance options
- Audit log storage settings
- Code ownership analysis configuration

### 5. Setup Database Storage

The framework requires a PostgreSQL database for audit storage:

```bash
# Create database
createdb -U postgres unified_compliance_audit

# Import schema
psql -U postgres -d unified_compliance_audit -f /path/to/llm_platform_recipe/schema/audit_schema.sql
```

Update your Drupal settings.php file to include the database connection:

```php
$databases['audit']['default'] = [
  'database' => 'unified_compliance_audit',
  'username' => 'postgres',
  'password' => 'your-password',
  'host' => 'localhost',
  'port' => '5432',
  'driver' => 'pgsql',
  'prefix' => '',
];
```

### 6. Configure Redis Cache

The framework uses Redis for caching security decisions and compliance status:

```php
// In settings.php
$settings['redis.connection']['interface'] = 'PhpRedis';
$settings['redis.connection']['host'] = '127.0.0.1';
$settings['redis.connection']['port'] = 6379;
$settings['cache']['default'] = 'cache.backend.redis';
$settings['container_yamls'][] = 'modules/contrib/redis/redis.services.yml';
```

### 7. Setup Initial Compliance Configuration

Run the initial setup script to configure your compliance frameworks:

```bash
drush ucf:setup --frameworks=GDPR,HIPAA,CODE_OWNERSHIP --default-policies
```

### 8. Verify Installation

Verify that the installation is working correctly:

```bash
# Check Drupal module status
drush pm-info unified_compliance_security

# Check bridge service status
sudo systemctl status unified-compliance-bridge

# Test security logging
drush ucf:test-audit-log

# Test compliance framework
drush ucf:compliance-status
```

## Configuration Options

### Security Configuration

The key security settings include:

- **Audit Level**: Determines the verbosity of audit logging (basic, standard, detailed)
- **Audit Retention**: How long to keep audit logs (30 days, 90 days, 1 year, 7 years)
- **Real-time Alerting**: Configure alerts for security violations
- **RBAC Integration**: How to integrate with Drupal's role system

### Compliance Configuration

Configure compliance frameworks:

- **Active Frameworks**: Which compliance frameworks to enforce
- **Validation Frequency**: How often to validate compliance
- **Automated Remediation**: Whether to automatically fix compliance issues
- **Reporting Schedule**: When to generate compliance reports

### Code Ownership Configuration

Configure code ownership analysis:

- **Analysis Depth**: How deep to analyze repository history
- **Knowledge Threshold**: When to consider knowledge concentration a risk
- **Integration Repos**: Which repositories to include in analysis
- **Review Strategy**: How to enforce review requirements

## Next Steps

After installation:

1. Review the [Usage Guide](./usage) for common operations
2. Set up the [Security Dashboard](./security-dashboard)
3. Configure your first [Compliance Framework](./compliance-frameworks)
4. Learn about [Plugin Development](./plugin-development)

## Troubleshooting

### Common Issues

1. **Bridge Connection Failed**
   
   If Drupal cannot connect to the Node.js bridge:
   
   - Check that the bridge service is running
   - Verify the socket connection settings
   - Check for firewall rules blocking connections
   - Review logs at `/var/log/unified-compliance-bridge.log`

2. **Audit Logging Failed**
   
   If audit logs are not being recorded:
   
   - Verify PostgreSQL connection settings
   - Check database permissions
   - Ensure the audit schema is properly installed
   - Review Drupal error logs

3. **Code Ownership Analysis Failed**
   
   If code ownership analysis isn't working:
   
   - Verify GitLab/GitHub API tokens
   - Check repository access permissions
   - Ensure git is installed and accessible
   - Review Node.js bridge logs

For additional help, contact your system administrator or open an issue in the GitLab repository.
