---
id: user-monitoring
---
# Monitoring System User Guide

## Overview

The monitoring system provides comprehensive system monitoring capabilities through an easy-to-use command-line interface. This guide will help you get started with monitoring your system resources, performance metrics, and system health.

## Quick Start

### Installation

```bash
npm install -g bfcli
```

### Basic Usage

To start monitoring system resources:

```bash
bfcli monitor resources
```

To monitor specific services:

```bash
bfcli monitor service --name nginx
```

To check system health:

```bash
bfcli monitor health
```

## Command Reference

### Resources Monitoring

Monitor system resources including CPU, memory, disk, and network usage.

```bash
bfcli monitor resources [options]

Options:
  --interval <seconds>  Set monitoring interval (default: 5)
  --metrics <list>      Comma-separated list of metrics to monitor
  --format <type>       Output format (json|table|csv)
  --output <file>       Save output to file
```

Example:
```bash
# Monitor CPU and memory every 2 seconds
bfcli monitor resources --interval 2 --metrics cpu,memory

# Save monitoring data to CSV file
bfcli monitor resources --format csv --output metrics.csv
```

### Performance Monitoring

Monitor system performance with customizable alerts.

```bash
bfcli monitor performance [options]

Options:
  --threshold <number>  Alert threshold (0-100)
  --alert <type>       Alert type (email|slack|webhook)
  --config <file>      Custom configuration file
```

Example:
```bash
# Monitor with 80% CPU threshold
bfcli monitor performance --threshold 80 --alert slack

# Use custom configuration
bfcli monitor performance --config ./monitor-config.json
```

### Health Checks

Monitor system health status and components.

```bash
bfcli monitor health [options]

Options:
  --detailed           Show detailed health information
  --components <list>  Specific components to check
  --timeout <seconds>  Health check timeout
```

Example:
```bash
# Detailed health check of specific components
bfcli monitor health --detailed --components database,cache

# Quick health status
bfcli monitor health
```

### Service Monitoring

Monitor specific services and their metrics.

```bash
bfcli monitor service [options]

Options:
  --name <service>     Service name to monitor
  --metrics <list>     Service-specific metrics
  --interval <seconds> Monitoring interval
```

Example:
```bash
# Monitor nginx with custom metrics
bfcli monitor service --name nginx --metrics connections,requests

# Monitor multiple services
bfcli monitor service --name "nginx,redis" --interval 10
```

### Network Monitoring

Monitor network traffic and connections.

```bash
bfcli monitor network [options]

Options:
  --interface <name>   Network interface to monitor
  --protocol <type>    Protocol filter (tcp|udp|all)
  --port <number>      Port filter
```

Example:
```bash
# Monitor specific interface
bfcli monitor network --interface eth0 --protocol tcp

# Monitor specific port
bfcli monitor network --port 80
```

## Configuration

### Default Configuration

The monitoring system uses a default configuration file located at `~/.bfcli/monitor-config.json`:

```json
{
  "global": {
    "interval": 5,
    "logLevel": "info"
  },
  "metrics": {
    "cpu": {
      "enabled": true,
      "threshold": 80
    },
    "memory": {
      "enabled": true,
      "threshold": 90
    },
    "disk": {
      "enabled": true,
      "threshold": 85
    }
  },
  "alerts": {
    "email": {
      "enabled": false,
      "recipients": []
    },
    "slack": {
      "enabled": false,
      "webhook": ""
    }
  }
}
```

### Custom Configuration

Create a custom configuration file:

1. Copy the default configuration:
   ```bash
   cp ~/.bfcli/monitor-config.json ./my-monitor-config.json
   ```

2. Edit the configuration file:
   ```bash
   vim my-monitor-config.json
   ```

3. Use the custom configuration:
   ```bash
   bfcli monitor resources --config ./my-monitor-config.json
   ```

## Alert Configuration

### Email Alerts

Configure email alerts in your configuration file:

```json
{
  "alerts": {
    "email": {
      "enabled": true,
      "recipients": ["admin@example.com"],
      "smtp": {
        "host": "smtp.example.com",
        "port": 587,
        "secure": true,
        "auth": {
          "user": "alerts@example.com",
          "pass": "YOUR_PASSWORD"
        }
      }
    }
  }
}
```

### Slack Alerts

Configure Slack alerts:

```json
{
  "alerts": {
    "slack": {
      "enabled": true,
      "webhook": "https://hooks.slack.com/services/YOUR/WEBHOOK/URL",
      "channel": "#monitoring",
      "username": "Monitor Bot"
    }
  }
}
```

## Best Practices

1. **Resource Usage**
   - Start with longer intervals (5-15 seconds) for regular monitoring
   - Use shorter intervals only when debugging specific issues
   - Monitor only the metrics you need

2. **Alert Configuration**
   - Set appropriate thresholds based on your system's normal behavior
   - Use different alert levels for different severity issues
   - Configure alert aggregation to prevent alert fatigue

3. **Data Management**
   - Regularly rotate log files
   - Archive historical data
   - Use appropriate data retention policies

4. **Security**
   - Use secure connections for remote monitoring
   - Protect sensitive configuration data
   - Regularly update authentication credentials

## Troubleshooting

### Common Issues

1. **High CPU Usage**
   ```bash
   # Check process details
   bfcli monitor resources --metrics cpu --detailed
   ```

2. **Memory Leaks**
   ```bash
   # Monitor memory trends
   bfcli monitor resources --metrics memory --interval 60
   ```

3. **Disk Space Issues**
   ```bash
   # Check disk usage details
   bfcli monitor resources --metrics disk --detailed
   ```

### Debug Mode

Enable debug mode for detailed logging:

```bash
bfcli monitor resources --debug
```

### Support

For additional support:

1. Check the logs:
   ```bash
   bfcli monitor logs
   ```

2. Generate a diagnostic report:
   ```bash
   bfcli monitor diagnostic
   ```

3. Contact support with the diagnostic report

## Advanced Usage

### Custom Metrics

Create custom metrics by extending the base monitor:

```typescript
import { MonitorBase } from '@bfcli/monitor';

class CustomMonitor extends MonitorBase {
  protected async collect(): Promise<void> {
    // Implement custom metric collection
    const metrics = await this.getCustomMetrics();
    this.emit('metrics', metrics);
  }
}
```

### Automation

Create monitoring scripts:

```bash
#!/bin/bash
# daily-monitor.sh
bfcli monitor resources --output daily-$(date +%Y%m%d).csv
bfcli monitor health --detailed > health-$(date +%Y%m%d).log
```

### Integration

Integrate with other tools:

```bash
# Send metrics to external system
bfcli monitor resources --format json | curl -X POST \
  -H "Content-Type: application/json" \
  -d @- http://metrics.example.com/collect
``` 
